/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network
 * Copyright (c) 2004-2016 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    HTTP_Server.c
 * Purpose: HTTP Server example
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include "cmsis_os.h"                   /* CMSIS RTOS definitions             */
#include "rl_net.h"                     /* Network definitions                */
#include "sam.h"                        // Device header
#include "Board_LED.h"                  // ::Board Support:LED
#include "Board_Buttons.h"              // ::Board Support:Buttons

extern void _SetupMemoryRegion( void );

bool LEDrun;

static void BlinkLed (void const *arg);

osThreadDef(BlinkLed, osPriorityNormal, 1, 0);

/// Read digital inputs
uint8_t get_button (void) {
  return (Buttons_GetState ());
}

/// IP address change notification
void dhcp_client_notify (uint32_t if_num,
                         dhcpClientOption opt, const uint8_t *val, uint32_t len) {            
  if (opt == dhcpClientIPaddress) {
    printf("IP: %d.%d.%d.%d\n",val[0],val[1],val[2],val[3]);
  }
}

/*----------------------------------------------------------------------------
  Thread 'BlinkLed': Blink the LEDs on an eval board
 *---------------------------------------------------------------------------*/
static void BlinkLed (void const *arg) {
  const uint8_t led_val[] = { 0x01, 0x02 };
  int cnt = 0;

  LEDrun = true;
  while(1) {
    // Every 200 ms
    if (LEDrun == true) {
      LED_SetOut (led_val[cnt]);
      if (++cnt >= sizeof(led_val)) {
        cnt = 0;
      }
    }
    osDelay (200);
  }
}

/*----------------------------------------------------------------------------
  Main Thread 'main': Run Network
 *---------------------------------------------------------------------------*/
int main (void) {
  
  /* Disable watchdog */
  WDT->WDT_MR = WDT_MR_WDDIS; 
  
  /* Setup MPU */
  _SetupMemoryRegion();
  
  /* Update system clock */
  SystemCoreClockUpdate();
  
  /* Enable cache */
  SCB_EnableDCache(); 
  SCB_EnableICache();   
    
  /* Leds initialization */
  LED_Initialize();
     
  /* Buttons initialization */
  Buttons_Initialize();
  
  /* Network initialization */
  net_initialize();

  osThreadCreate (osThread(BlinkLed), NULL);

  while(1) {
    net_main ();
    osThreadYield ();
  }
}
