/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network
 * Copyright (c) 2004-2016 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    HTTP_Upload.c
 * Purpose: HTTP File upload example
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include "cmsis_os.h"                   /* CMSIS RTOS definitions             */
#include "rl_net.h"                     /* Network definitions                */
#include "rl_fs.h"                      /* FileSystem definitions             */

#include "sam.h"                        // Device header
#include "Board_LED.h"                  // ::Board Support:LED
#include "Board_Buttons.h"              // ::Board Support:Buttons

extern void _SetupMemoryRegion( void );

static void BlinkLed (void const *arg);

osThreadDef(BlinkLed, osPriorityNormal, 1, 0);

// IP address change notification
void dhcp_client_notify (uint32_t if_num,
                         dhcpClientOption opt, const uint8_t *val, uint32_t len) {
  if (opt == dhcpClientIPaddress) {
    printf("IP: %d.%d.%d.%d\n",val[0],val[1],val[2],val[3]);
  }
}
                         
// Open a file for reading in HTTP server.
void *http_server_fopen (const char *fname) {
char str[256] = {0};
char *p = str;
char symb[3] = {0};
uint32_t c;

  // Parse URL-encoding
  while(*fname){
    if (*fname == '%') {
      memcpy(symb,fname+1,2);
      sscanf(symb,"%x",&c);
      *p++ = c;
      fname += 3;
    } else {
      *p++ = *fname++;
    }
  }
  return (fopen (str, "r"));
}

/*----------------------------------------------------------------------------
  Thread 'BlinkLed': Blink the LEDs on an eval board
 *---------------------------------------------------------------------------*/
static void BlinkLed (void const *arg) {
  const uint8_t led_val[] = { 0x01U ,0x02U };
  int cnt = 0u;

  while(1) {
    // Every 200 ms
    LED_SetOut (led_val[cnt]);
    if (++cnt >= sizeof(led_val)) {
      cnt = 0U;
    }
    osDelay (200U);
  }
}

/*----------------------------------------------------------------------------
  Main Thread 'main': Run Network
 *---------------------------------------------------------------------------*/
int main (void) {
  
  /* Disable watchdog */
  WDT->WDT_MR = WDT_MR_WDDIS; 
  
  /* Setup MPU */
  _SetupMemoryRegion();
  
  /* Update system clock */
  SystemCoreClockUpdate();
    
  /* Enable cache */
  SCB_EnableDCache(); 
  SCB_EnableICache();   
      
  /* Leds initialization */
  LED_Initialize();
 
  /* Buttons initialization */
  Buttons_Initialize();
  
  /* FileSystem initialization */
  if (finit ("M0:") == fsOK) {
    fmount ("M0:");
  }

  /* Network initialization */
  net_initialize();

  osThreadCreate (osThread(BlinkLed), NULL);

  while(1) {
    net_main ();
    osThreadYield ();
  }
}
