/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network:Service
 * Copyright (c) 2004-2014 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    Telnet_Server_UIF.c
 * Purpose: Telnet Server User Interface
 * Rev.:    V6.00
 *----------------------------------------------------------------------------*/
//! [code_Telnet_Server_UIF]
#include <stdio.h>
#include <string.h>
#include "rl_net.h"
#include "rl_net_lib.h"
#include "Board_LED.h"                  // ::Board Support:LED
 
extern struct tnet_cfg  tnet_config;
#define tnet_EnAuth     tnet_config.EnAuth
#define tnet_auth_passw tnet_config.Passw
 
 // Local variables
static const char telnet_help1[] = {
  "\r\n\r\n"
  "Commands:\r\n"
  "  led xx      - write hexval xx to LED port\r\n"
};
static const char telnet_help2[] = {
  "  passw [new] - change system password\r\n"
  "  passwd      - display current password\r\n"
};
static const char telnet_help3[] = {
  "  help, ?     - display this help\r\n"
  "  bye,\r\n"
  "  <ESC>,<^C>  - disconnect\r\n"
  "  <BS>        - delete Character left\r\n"
  "  <UP>,<DOWN> - recall Command History\r\n"
};


// Request message for Telnet server session.
uint32_t telnet_server_message (telnetServerMessage msg, char *buf, uint32_t len) {
  uint32_t rlen = 0;
 
  switch (msg) {
    case telnetServerWelcome:
      // Initial welcome message
      rlen = sprintf (buf, "\r\n"
                           "\r\n"
                           "Telnet Server ready\r\n");

      break;
    case telnetServerPrompt:
      // Prompt message
      rlen = sprintf (buf, "\r\n"
                           "Cmd> ");
      break;
    case telnetServerLogin:
      // Login welcome message, if authentication is enabled
      rlen = sprintf (buf, "\r\n"
                           "Embedded Telnet Server\r\n"
                           "\r\n"
                           "Please login...");
      break;
    case telnetServerUsername:
      // Username request login message
      rlen = sprintf (buf, "\r\n"
                           "Username: ");
      break;
    case telnetServerPassword:
      // Password request login message
      rlen = sprintf (buf, "\r\n"
                           "Password: ");
      break;
    case telnetServerLoginFailed:
      // Incorrect login error message
      rlen = sprintf (buf, "\r\n"
                           "Login incorrect");
      break;
    case telnetServerLoginTimeout:
      // Login timeout error message
      rlen = sprintf (buf, "\r\n"
                           "Login timeout\r\n");
      break;
    case telnetServerUnsolicitedMessage:
      // Unsolicited message (ie. from basic interpreter)
      rlen = sprintf (buf, "\r\n"
                           "Unsolicited message\r\n");
      break;
  }
  return (rlen);
}
 
// Poll the upper-layer user application for unsolicited messages.
bool telnet_server_message_poll (int32_t session) {
  /*
  extern bool msg_available;
 
  if (session == 1 && msg_available == true) {
    // Allow unsolicited messages on 1st session
    msg_available = false;
    return (true);
  }
  */
  return (false);
}
 
// Process a command and generate response.
uint32_t telnet_server_process (const char *cmd, char *buf, uint32_t buflen, uint32_t *pvar) {
  uint32_t len = 0;
  uint32_t val;
    
  if (telnet_check_command (cmd, "PASSW") == true && tnet_EnAuth) {
    // PASSW command given, change password
    if (len == 5) {
      // Disable password
      tnet_auth_passw[0] = 0;
    }
    else {
      // Change password
      memcpy (tnet_auth_passw, &cmd[6], 20);
    }
    len = sprintf (buf, "\r\n OK, New Password: \"%s\"", tnet_auth_passw);
    return (len);
  }

  if (telnet_check_command (cmd, "PASSWD") == true && tnet_EnAuth) {
    // PASSWD command given, print current password
    len = sprintf (buf, "\r\n System Password: \"%s\"", tnet_auth_passw);
    return (len);
  }
  
  if (telnet_check_command (cmd, "LED") == true) {
    // LED command given, control onboard LEDs
    len = strlen (cmd);
    if (len >= 5) {
      sscanf (cmd+4,"%x", &val);
      LED_SetOut (val);
      len = sprintf (buf," --> LED state updated");
      return (len);
    }
  }

  if (telnet_check_command (cmd, "BYE") == true) {
    // Generate reply and disconnect
    len = sprintf (buf, "\r\n"
                        "Disconnecting\r\n");
    return (len | (1u<<30));
  }

  if (telnet_check_command (cmd, "HELP") == true ||
      telnet_check_command (cmd, "?") == true) {
    // HELP command given, display help text
    len = sprintf (buf, telnet_help1);
    if (tnet_EnAuth) {
      len += sprintf (buf+len, telnet_help2);
    }
    len += sprintf (buf+len, telnet_help3);
    return (len);
  }
      
  // Unknown command given
  len = sprintf (buf, "\r\n==> Unknown Command: %s", cmd);
  return (len);
}
//! [code_Telnet_Server_UIF]
