/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: LED Flasher for SAM4S-XPRO
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2004-2015 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include "sam.h"                        // Device header
#include "cmsis_os.h"                   // ARM::CMSIS:RTOS:Keil RTX
#include "Board_Buttons.h"              // ::Board Support:Buttons
#include "Board_LED.h"                  // ::Board Support:LED


/*----------------------------------------------------------------------------
  Main function
 *----------------------------------------------------------------------------*/
int main (void) {
  int32_t max_num = LED_GetCount() - 1;
  int32_t num = 0;
  int32_t dir = 1;

  /* Disable watchdog */
  WDT->WDT_MR = WDT_MR_WDDIS;
	
  /* Update system clock */
  SystemCoreClockUpdate();
  
  /* Enable data cache */
  SCB_EnableDCache();	
  SCB_EnableICache();
  
  LED_Initialize();                         /* LED Initialization             */
  Buttons_Initialize();                     /* Buttons Initialization         */

  while (1) {
    LED_On(num);                            /* Turn specified LED on          */
    osDelay(500);                           /* Wait 500ms                     */
    while (Buttons_GetState() & (1 << 0));  /* Wait while holding USER button */
    LED_Off(num);                           /* Turn specified LED off         */
    osDelay(500);                           /* Wait 500ms                     */
    while (Buttons_GetState() & (1 << 0));  /* Wait while holding USER button */

    num += dir;                             /* Change LED number              */
    if (dir == 1 && num == max_num) {
      dir = -1;                             /* Change direction to down       */
    }
    else if (num == 0) {
      dir =  1;                             /* Change direction to up         */
    }
  }
}
