/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network:Service
 * Copyright (c) 2004-2014 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    SMTP_Client_UIF.c
 * Purpose: SMTP Client User Interface
 * Rev.:    V6.00
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include "rl_net.h"
#include "rl_net_lib.h"

// Email definitions
#define MAIL_FROM       "mcb@keil.com"
#define RCPT_TO         "somebody@keil.com"
#define SMTP_USER       "user"
#define SMTP_PASSWORD   "password"
#define MAIL_SUBJECT    "Email from embedded SMTP Client"

#define MSG_BODY                                                      \
  "Hello!\r\n\r\n"                                                    \
  "This is an email sent from '%s' embedded system.\r\n"              \

// Net_Config.c 
extern struct sys_cfg   sys_config;
#define lhost_name      sys_config.HostName

// Functions
extern uint16_t AD_in (uint32_t ch);
extern void smtp_inform (smtpClientEvent event);

// Request parameters for SMTP client session.
uint32_t smtp_client_request (smtpClientRequest request, char *buf, uint32_t buflen, uint32_t *pvar) {
  uint32_t len = 0;

  switch (request) {
    case smtpClientUsername:
      // Username to login to SMTP server
      len = sprintf (buf, SMTP_USER);
      break;

    case smtpClientPassword:
      // Password to login to SMTP server
      len = sprintf (buf, SMTP_PASSWORD);
      break;

    case smtpClientSender:
      // Email address of the sender
      len = sprintf (buf, MAIL_FROM);
      break;

    case smtpClientRecipient:
      // Email address of the recipient
      len = sprintf (buf, RCPT_TO);
      break;

    case smtpClientSubject:
      // Subject of email
      len = sprintf (buf, MAIL_SUBJECT);
      break;

    case smtpClientBody:
      // Email body in plain ascii formatl
      len = sprintf (buf, MSG_BODY, lhost_name);
      break;
  }
  return (len);
}

// Notify the user application when SMTP client operation ends.
void smtp_client_notify (smtpClientEvent event) {
  smtp_inform (event);
}

// Accept or deny authentication requested by SMTP server.
bool smtp_client_accept_authentication (const uint8_t *ip_addr) {
  return (true);
}
