/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network
 * Copyright (c) 2004-2016 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    SNMP_Agent.c
 * Purpose: SNMP Agent example
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include "cmsis_os.h"                   /* CMSIS RTOS definitions            */
#include "rl_net.h"                     /* Network definitions               */
#include "sam.h"                        // Device header
#include "Board_LED.h"                  // ::Board Support:LED
#include "Board_Buttons.h"              // ::Board Support:Buttons

extern void _SetupMemoryRegion( void );

char text[2][20+1];

/// Read digital inputs
uint8_t get_button (void) {
  return (Buttons_GetState ());
}

/// IP address change notification
void dhcp_client_notify (uint32_t if_num,
                         dhcpClientOption opt, const uint8_t *val, uint32_t len) {                           
  if (opt == dhcpClientIPaddress) {
    printf("IP: %d.%d.%d.%d\n",val[0],val[1],val[2],val[3]);
  }
}

/*----------------------------------------------------------------------------
  Main Thread 'main': Run Network
 *---------------------------------------------------------------------------*/
int main (void) {
  
  /* Disable watchdog */
  WDT->WDT_MR = WDT_MR_WDDIS; 
  
  /* Setup MPU */
  _SetupMemoryRegion();
  
  /* Update system clock */
  SystemCoreClockUpdate();
  
  /* Enable cache */
  SCB_EnableDCache(); 
  SCB_EnableICache();   
    
  /* Leds initialization */
  LED_Initialize();
 
  /* Buttons initialization */
  Buttons_Initialize();
  
  /* Network initialization */
  net_initialize();

  while (1) {
    net_main ();
    osThreadYield ();
  }
}
