/*----------------------------------------------------------------------------
 * Name:    main.c
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2004-2016 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include "sam.h"                        // Device header
#include "cmsis_os.h"                   /* CMSIS RTOS definitions             */
#include "rl_net.h"                     /* Network definitions                */
#include "Board_LED.h"                  // ::Board Support:LED
#include <stdio.h>

extern void _SetupMemoryRegion( void );

// IP address change notification
void dhcp_client_notify (uint32_t if_num,
                         dhcpClientOption opt, const uint8_t *val, uint32_t len) {                  
  if (opt == dhcpClientIPaddress) {
    printf("IP: %d.%d.%d.%d",val[0],val[1],val[2],val[3]);
  }
}
                         

/*----------------------------------------------------------------------------
  Main function
 *----------------------------------------------------------------------------*/
int main (void) {

  /* Disable watchdog */
  WDT->WDT_MR = WDT_MR_WDDIS; 
  
  /* Setup MPU */
  _SetupMemoryRegion();
  
  /* Update system clock */
  SystemCoreClockUpdate();
  
  /* Enable cache */
  SCB_EnableDCache(); 
  SCB_EnableICache();   
    
  /* Leds initialization */
  LED_Initialize();
 
  /* Network initialization */
  net_initialize();
  
  while (1) {
    net_main ();
  }

}
