/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network
 * Copyright (c) 2004-2016 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    HTTP_Server.c
 * Purpose: HTTP Server example
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include "cmsis_os.h"                   /* CMSIS RTOS definitions             */
#include "rl_net.h"                     /* Network definitions                */

#include "sam.h"                     
#include "Board_LED.h"
#include "Board_Buttons.h"              

extern void _SetupMemoryRegion( void );

bool LEDrun;

// Thread IDs
osThreadId TID_DebugViewer;
osThreadId TID_Led;

// Thread definitions
static void BlinkLed (void const *arg);
static void DebugViewer  (void const *arg);

osThreadDef(BlinkLed, osPriorityNormal, 1, 0);
osThreadDef(DebugViewer,  osPriorityNormal, 1, 0);

/// Read digital inputs
uint8_t get_button (void) {
  return (Buttons_GetState ());
}

/// IP address change notification
void netDHCP_Notify (uint32_t if_num, uint8_t option, const uint8_t *val, uint32_t len) {

  if (option == NET_DHCP_OPTION_IP_ADDRESS) {
    /* IP address change, trigger LCD update */
    osSignalSet (TID_DebugViewer, 0x01);
  }
}

/*----------------------------------------------------------------------------
  Thread 'DebugViewer'
 *---------------------------------------------------------------------------*/
static void DebugViewer (void const *arg) {
  static uint8_t ip_addr[NET_ADDR_IP6_LEN];
  static char    ip_ascii[40];

  // Print Link-local IPv6 address
  netIF_GetOption (NET_IF_CLASS_ETH,
                   netIF_OptionIP6_LinkLocalAddress, ip_addr, sizeof(ip_addr));

  netIP_ntoa(NET_ADDR_IP6, ip_addr, ip_ascii, sizeof(ip_ascii));

  printf ("IP6:%.16s%s\n", ip_ascii, ip_ascii+16);

  while(1) {
    /* Wait for signal from DHCP */
    osSignalWait (0x01, osWaitForever);
    /* Retreive and print IPv4 address */
    netIF_GetOption (NET_IF_CLASS_ETH,
                     netIF_OptionIP4_Address, ip_addr, sizeof(ip_addr));

    netIP_ntoa (NET_ADDR_IP4, ip_addr, ip_ascii, sizeof(ip_ascii));

    printf ("IP4:%-16s\n",ip_ascii);
  }
}

/*----------------------------------------------------------------------------
  Thread 'BlinkLed': Blink the LEDs on an eval board
 *---------------------------------------------------------------------------*/
static void BlinkLed (void const *arg) {
  const uint8_t led_val[] = { 0x00, 0x01 };
  int cnt = 0;

  LEDrun = true;
  while(1) {
    // Every 200 ms
    if (LEDrun == true) {
      LED_SetOut (led_val[cnt]);
      if (++cnt >= sizeof(led_val)) {
        cnt = 0;
      }
    }
    osDelay (200);
  }
}

/*----------------------------------------------------------------------------
  Main Thread 'main': Run Network
 *---------------------------------------------------------------------------*/
int main (void) {
  
  /* Disable watchdog */
  WDT->WDT_MR = WDT_MR_WDDIS; 
  
  /* Setup MPU */
  _SetupMemoryRegion();
  
  /* Update system clock */
  SystemCoreClockUpdate();
  
  /* Enable cache */
  SCB_EnableDCache(); 
  SCB_EnableICache();   
    
  /* Leds and buttons initialization */
  LED_Initialize();
  Buttons_Initialize ();
 
  /* Network initialization */
  netInitialize();

  TID_Led = osThreadCreate (osThread(BlinkLed), NULL);
  TID_DebugViewer = osThreadCreate (osThread(DebugViewer), NULL);

  while(1) {
    osSignalWait (0, osWaitForever);
  }
}
