/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network:Service
 * Copyright (c) 2004-2015 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    SMTP_Client_UIF.c
 * Purpose: SMTP Client User Interface
 * Rev.:    V6.00
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include "rl_net.h"                     // Keil.MDK-Pro::Network:CORE

// Email definitions
#define MAIL_FROM       "mcb@keil.com"
#define RCPT_TO         "somebody@keil.com"
#define SMTP_USER       "user"
#define SMTP_PASSWORD   "password"
#define MAIL_SUBJECT    "Email from embedded SMTP Client"

#define MSG_BODY                                                      \
  "Hello!\r\n\r\n"                                                    \
  "This is an email sent from '%s' embedded system.\r\n"              \

// Functions
extern void smtp_inform (netSMTPc_Event event);

// Request parameters for SMTP client session.
uint32_t netSMTPc_Process (netSMTPc_Request request, char *buf, uint32_t buflen, uint32_t *pvar) {
  uint32_t len = 0;

  switch (request) {
    case netSMTPc_RequestUsername:
      // Username to login to SMTP server
      len = sprintf (buf, SMTP_USER);
      break;

    case netSMTPc_RequestPassword:
      // Password to login to SMTP server
      len = sprintf (buf, SMTP_PASSWORD);
      break;

    case netSMTPc_RequestSender:
      // Email address of the sender
      len = sprintf (buf, MAIL_FROM);
      break;

    case netSMTPc_RequestRecipient:
      // Email address of the recipient
      len = sprintf (buf, RCPT_TO);
      break;

    case netSMTPc_RequestSubject:
      // Subject of email
      len = sprintf (buf, MAIL_SUBJECT);
      break;

    case netSMTPc_RequestBody:
      // Email body in plain ascii formatl
      len = sprintf (buf, MSG_BODY, netSYS_GetHostName());
      break;
  }
  return (len);
}

// Notify the user application when SMTP client operation ends.
void netSMTPc_Notify  (netSMTPc_Event event) {
  smtp_inform (event);
}

// Accept or deny authentication requested by SMTP server.
bool netSMTPc_AcceptAuthentication (const NET_ADDR *addr) {
  return (true);
}
