/* -----------------------------------------------------------------------------
 * Copyright (c) 2015 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * $Date:        27. August 2015
 * $Revision:    V1.02
 *  
 * Project:      CMSIS I2C Driver for Atmel SAM-ESV7
 * --------------------------------------------------------------------------*/

#include "I2C_SAM-ESV7.h"

#define XDMA_INT (XDMAC_CIE_BIE | XDMAC_CIE_DIE | XDMAC_CIE_FIE | XDMAC_CIE_RBIE | XDMAC_CIE_WBIE | XDMAC_CIE_ROIE)

#define ARM_I2C_DRV_VERSION ARM_DRIVER_VERSION_MAJOR_MINOR(1,02) /* driver version */

/* Driver Version */
static const ARM_DRIVER_VERSION DriverVersion = {
  ARM_I2C_API_VERSION,
  ARM_I2C_DRV_VERSION
};

/* Driver Capabilities */
static const ARM_I2C_CAPABILITIES DriverCapabilities =
{
  0 // supports 10-bit addressing
};

#if (defined(RTE_Drivers_I2C) &&                \
  (RTE_I2C0 == 0) &&                         \
  (RTE_I2C1 == 0) &&                         \
  (RTE_I2C2 == 0))
#error "I2C not configured in RTE_Device.h"
#endif

// Driver mode definition
#if   ((RTE_I2C0 != 0) && defined(RTE_I2C0_MODE_IRQ))|| \
  ((RTE_I2C1 != 0) && defined(RTE_I2C1_MODE_IRQ))||   \
  ((RTE_I2C2 != 0) && defined(RTE_I2C2_MODE_IRQ))
#define I2C_MODE_IRQ

#elif ((RTE_I2C0 != 0) && defined(RTE_I2C0_MODE_POLLING))||     \
  ((RTE_I2C1 != 0) && defined(RTE_I2C1_MODE_POLLING))||       \
  ((RTE_I2C2 != 0) && defined(RTE_I2C2_MODE_POLLING))
#define I2C_MODE_POLLING

#endif

/* I2C0 */
#if (RTE_I2C0 != 0)

// I2C0 Status
static I2C_INFO I2C0_Info = {0};
static I2C_TRANSFER_INFO I2C0_TransferInfo = {0};

// I2C0 Configure pins
#if RTE_I2C0_SCL_PORT_ID == 1
static Pin I2C0_SCL_PORT        = RTE_I2C0_SCL_PORT;
#endif
#if RTE_I2C0_SDA_PORT_ID == 1
static Pin I2C0_SDA_PORT        = RTE_I2C0_SDA_PORT;
#endif

static I2C_IO I2C0_Pins = {
#if RTE_I2C0_SCL_PORT_ID == 1
  &I2C0_SCL_PORT,
#else
  NULL,
#endif
#if RTE_I2C0_SDA_PORT_ID == 1
  &I2C0_SDA_PORT,
#else
  NULL,
#endif
};


// I2C0 Resources
static I2C_RESOURCES I2C0_Resources = {
/* Driver Capabilities */
  {
    0 // supports 10-bit addressing
  },
  TWIHS0,
  ID_TWIHS0,
  TWIHS0_IRQn,
  &I2C0_Pins,
  &I2C0_Info,
  &I2C0_TransferInfo,
  NULL,
  NULL,
  NULL,
};

#endif /* RTE_I2C0 */

/* I2C1 */
#if (RTE_I2C1 != 0)

// I2C1 Status
static I2C_INFO I2C1_Info = {0};
static I2C_TRANSFER_INFO I2C1_TransferInfo = {0};

// I2C1 Configure pins
#if RTE_I2C1_SCL_PORT_ID == 1
static Pin I2C1_SCL_PORT        = RTE_I2C1_SCL_PORT;
#endif
#if RTE_I2C1_SDA_PORT_ID == 1
static Pin I2C1_SDA_PORT        = RTE_I2C1_SDA_PORT;
#endif

static I2C_IO I2C1_Pins = {
#if RTE_I2C1_SCL_PORT_ID == 1
  &I2C1_SCL_PORT,
#else
  NULL,
#endif
#if RTE_I2C1_SDA_PORT_ID == 1
  &I2C1_SDA_PORT,
#else
  NULL,
#endif
};

// I2C1 Resources
static I2C_RESOURCES I2C1_Resources = {
  /* Driver Capabilities */
  {
    0 // supports 10-bit addressing
  },
  TWIHS1,
  ID_TWIHS1,
  TWIHS1_IRQn,
  &I2C1_Pins,
  &I2C1_Info,
  &I2C1_TransferInfo,
  NULL,
  NULL,
  NULL,
};

#endif /* RTE_I2C1 */

/* I2C2 */
#if (RTE_I2C2 != 0)

// I2C2 Status
static I2C_INFO I2C2_Info = {0};
static I2C_TRANSFER_INFO I2C2_TransferInfo = {0};

// I2C2 Configure pins
#if RTE_I2C2_SCL_PORT_ID == 1
static Pin I2C2_SCL_PORT        = RTE_I2C2_SCL_PORT;
#endif
#if RTE_I2C2_SDA_PORT_ID == 1
static Pin I2C2_SDA_PORT        = RTE_I2C2_SDA_PORT;
#endif

static I2C_IO I2C2_Pins = {
#if RTE_I2C2_SCL_PORT_ID == 1
  &I2C2_SCL_PORT,
#else
  NULL,
#endif
#if RTE_I2C2_SDA_PORT_ID == 1
  &I2C2_SDA_PORT,
#else
  NULL,
#endif
};

// I2C2 Resources
static I2C_RESOURCES I2C2_Resources = {
  /* Driver Capabilities */
  {
    0 // supports 10-bit addressing
  },
  TWIHS2,
  ID_TWIHS2,
  TWIHS2_IRQn,
  &I2C2_Pins,
  &I2C2_Info,
  &I2C2_TransferInfo,
  NULL,
  NULL,
  NULL,
};

#endif /* RTE_I2C2 */


/* I2C IRQ Handler */
static void I2Cx_IRQHandler (const I2C_RESOURCES *i2c) {
  uint32_t event = 0;
  uint32_t status = 0;

  status = i2c->reg->TWIHS_SR;
  status &= i2c->reg->TWIHS_IMR;

  if ((status & TWIHS_SR_SVACC) == TWIHS_SR_SVACC) {
    if (i2c->xfer->cnt < i2c->xfer->num) {
      if ((i2c->reg->TWIHS_SR & TWIHS_SR_SVREAD) == TWIHS_SR_SVREAD) {
        status = i2c->reg->TWIHS_SR;
        if ((status & TWIHS_SR_TXRDY) == TWIHS_SR_TXRDY) {
          i2c->reg->TWIHS_THR = i2c->xfer->buf[i2c->xfer->cnt++];
        } else {
          if ((status & TWIHS_SR_RXRDY) == TWIHS_SR_RXRDY)
            i2c->xfer->buf[i2c->xfer->cnt++] = i2c->reg->TWIHS_RHR;
        }
      }
    }

    if (i2c->xfer->cnt >= i2c->xfer->num) {
      i2c->reg->TWIHS_IDR = TWIHS_IDR_SVACC | TWIHS_IDR_RXRDY | TWIHS_IDR_TXRDY;
      i2c->reg->TWIHS_IER = TWIHS_IER_TXCOMP;
          
      /* event |= ARM_I2C_EVENT_TRANSFER_DONE; */
      /* i2c->info->cb_event(event); */
      /* i2c->info->status.busy = 0U; */
    }
    
  } else {
    
    if (TWI_STATUS_RXRDY(status)) {
      i2c->xfer->buf[i2c->xfer->cnt++] = i2c->reg->TWIHS_RHR;
  
      /* check for transfer finish */
      if (i2c->xfer->cnt == i2c->xfer->num) {
        i2c->reg->TWIHS_IDR = TWIHS_IDR_RXRDY;

        if ((i2c->xfer->ctrl & XFER_CTRL_XPENDING) != XFER_CTRL_XPENDING) {
          i2c->reg->TWIHS_IER = TWIHS_IER_TXCOMP;
          i2c->xfer->ctrl &= ~XFER_CTRL_XPENDING;
                
        }
        event |= ARM_I2C_EVENT_TRANSFER_DONE;
        i2c->info->cb_event(event);
        i2c->info->status.busy = 0U;
      }
      /* Last byte? */
      else if (i2c->xfer->cnt == (i2c->xfer->num - 1)) {
        if ((i2c->xfer->ctrl & XFER_CTRL_XPENDING) != XFER_CTRL_XPENDING) {
          i2c->reg->TWIHS_CR = TWIHS_CR_STOP;
        }
      }
    } else if (TWI_STATUS_TXRDY(status)) {
      i2c->reg->TWIHS_THR = i2c->xfer->buf[i2c->xfer->cnt++];
  
      /* Transfer finished ? */
      if (i2c->xfer->cnt == i2c->xfer->num) {

        i2c->reg->TWIHS_IDR = TWIHS_IDR_TXRDY;

        if ((i2c->xfer->ctrl & XFER_CTRL_XPENDING) != XFER_CTRL_XPENDING) {
          i2c->reg->TWIHS_IER = TWIHS_IER_TXCOMP;
          i2c->reg->TWIHS_CR = TWIHS_CR_STOP;
          i2c->xfer->ctrl &= ~XFER_CTRL_XPENDING;
        }
        event |= ARM_I2C_EVENT_TRANSFER_DONE;
        i2c->info->cb_event(event);
        i2c->info->status.busy = 0U;
      }
    }
  }

  if (TWI_STATUS_TXCOMP(status)) {
    i2c->reg->TWIHS_IDR = TWIHS_IDR_TXCOMP;
    event |= ARM_I2C_EVENT_TRANSFER_DONE;
    i2c->info->cb_event(event);
    i2c->info->status.busy = 0U;
  }
}


/**
   \fn          ARM_DRIVER_VERSION I2Cx_GetVersion (void)
   \brief       Get driver version.
   \return      \ref ARM_DRIVER_VERSION
*/
ARM_DRIVER_VERSION I2Cx_GetVersion(void)
{
  return DriverVersion;
}

/**
   \fn          ARM_I2C_CAPABILITIES I2Cx_GetCapabilities (const I2C_RESOURCES *i2c)
   \brief       Get driver capabilities
   \param[in]   i2c     Pointer to I2C resources
   \return      \ref ARM_I2C_CAPABILITIES
*/
ARM_I2C_CAPABILITIES I2Cx_GetCapabilities(const I2C_RESOURCES *i2c)
{
  return DriverCapabilities;
}

/**
   \fn          int32_t I2Cx_Initialize (      ARM_I2C_SignalEvent_t  cb_event
   const I2C_RESOURCES         *i2c)
   \brief       Initialize I2C Interface.
   \param[in]   cb_event  Pointer to \ref ARM_I2C_SignalEvent
   \param[in]   i2c     Pointer to I2C resources
   \return      \ref execution_status
*/
int32_t I2Cx_Initialize(ARM_I2C_SignalEvent_t cb_event, const I2C_RESOURCES *i2c)
{
  if (i2c->info->state & I2C_POWERED) {
    // Device is powered - could not be re-initialized
    return ARM_DRIVER_ERROR;
  }

  if (i2c->info->state & I2C_INITIALIZED) {
    // Driver is already initialized
    return ARM_DRIVER_OK;
  }

  // Initialize I2C Run-time Resources
  i2c->info->cb_event = cb_event;

  i2c->info->status.busy             = 0U;
  i2c->info->status.mode             = 0U;
  i2c->info->status.direction        = 0U;
  i2c->info->status.general_call     = 0U;
  i2c->info->status.arbitration_lost = 0U;
  i2c->info->status.bus_error        = 0U;

  // Clear transfer information
  memset(i2c->xfer, 0, sizeof(I2C_TRANSFER_INFO));

    // Pin configuration
  if (i2c->id == ID_TWIHS2) {
    // Disable the MATRIX registers write protection
    MATRIX->MATRIX_WPMR  = MATRIX_WPMR_WPKEY_PASSWD;
    MATRIX->CCFG_SYSIO |= CCFG_SYSIO_SYSIO4 | CCFG_SYSIO_SYSIO5;
  }

  if (i2c->io->scl)
    PIO_Configure(i2c->io->scl, 1);
  if (i2c->io->sda)
    PIO_Configure(i2c->io->sda, 1);

  PMC_EnablePeripheral(i2c->id);
  /* Disable I2C */
  i2c->reg->TWIHS_CR = TWIHS_CR_SVEN;
  i2c->reg->TWIHS_CR = TWIHS_CR_SWRST;
  i2c->reg->TWIHS_RHR;
  i2c->reg->TWIHS_CR  = TWIHS_CR_MSDIS | TWIHS_CR_SVDIS;
  
  i2c->info->state = I2C_INITIALIZED;

  return ARM_DRIVER_OK;
}

/**
   \fn          int32_t I2Cx_Uninitialize (const I2C_RESOURCES *i2c)
   \brief       De-initialize I2C Interface.
   \param[in]   i2c     Pointer to I2C resources
   \return      \ref execution_status
*/
int32_t I2Cx_Uninitialize(const I2C_RESOURCES *i2c)
{
  i2c->info->state = 0; // I2C is uninitiliazed

  return ARM_DRIVER_OK;
}

/**
   \fn          int32_t I2Cx_PowerControl (ARM_POWER_STATE state, I2C_RESOURCES *i2c)
   \brief       Control I2C Interface Power.
   \param[in]   state  Power state
   \param[in]   i2c  Pointer to I2C resources
   \return      \ref execution_status
*/
static int32_t I2Cx_PowerControl (ARM_POWER_STATE state, I2C_RESOURCES *i2c) {
    
  switch (state) {
    case ARM_POWER_OFF: 
    // Disable I2C Interrupts
    NVIC_ClearPendingIRQ(i2c->irq_num);
    NVIC_DisableIRQ(i2c->irq_num);            
    /* Disable I2C */
    i2c->reg->TWIHS_CR  = TWIHS_CR_MSDIS | TWIHS_CR_SVDIS;
    i2c->reg->TWIHS_IDR = TWIHS_IDR_RXRDY | TWIHS_IDR_TXRDY | TWIHS_IDR_TXCOMP;
    PMC_DisablePeripheral(i2c->id);
      i2c->info->state &= ~I2C_POWERED;
      break;

    case ARM_POWER_LOW:
        return ARM_DRIVER_ERROR_UNSUPPORTED;

    case ARM_POWER_FULL:
      if ((i2c->info->state & I2C_INITIALIZED) == 0) {
        /* Driver not initialized */
        return (ARM_DRIVER_ERROR);
      }
      
      PMC_EnablePeripheral(i2c->id);
      i2c->reg->TWIHS_CR = TWIHS_CR_SWRST;
      i2c->reg->TWIHS_RHR;
      i2c->reg->TWIHS_CR = TWIHS_CR_SVDIS | TWIHS_CR_MSDIS;

#if defined(I2C_MODE_IRQ)
      // Configure and enable SPI interrupt
      NVIC_ClearPendingIRQ(i2c->irq_num);                                         
      NVIC_EnableIRQ(i2c->irq_num);
#endif

      /* Ready for operation */
      i2c->info->state |= I2C_POWERED;
      break;
  }

  return ARM_DRIVER_OK;
}

/**
   \fn          int32_t I2Cx_MasterTransmit (uint32_t       addr,
   const uint8_t *data,
   uint32_t       num,
   bool           xfer_pending,
   I2C_RESOURCES *i2c)
   \brief       Start transmitting data as I2C Master.
   \param[in]   addr          Slave address (7-bit or 10-bit)
   \param[in]   data          Pointer to buffer with data to send to I2C Slave
   \param[in]   num           Number of data bytes to send
   \param[in]   xfer_pending  Transfer operation is pending - Stop condition will not be generated
   \param[in]   i2c           Pointer to I2C resources
   \return      \ref execution_status
*/
static int32_t I2Cx_MasterTransmit (uint32_t       addr,
                                    const uint8_t *data,
                                    uint32_t       num,
                                    bool           xfer_pending,
                                    I2C_RESOURCES *i2c) {
  uint32_t event = 0;
  uint32_t status = 0;

  if ((data == NULL) || (num == 0U)) {
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if ((addr & ~(ARM_I2C_ADDRESS_10BIT | ARM_I2C_ADDRESS_GC)) > 0x3FFU) {
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if (i2c->info->status.busy) {
    return (ARM_DRIVER_ERROR_BUSY);
  }

  i2c->info->status.busy             = 1U;
  i2c->info->status.mode             = 1U;
  i2c->info->status.direction        = 0U;
  i2c->info->status.bus_error        = 0U;
  i2c->info->status.arbitration_lost = 0U;
  
  // Save transmit buffer info
  i2c->xfer->buf = (uint8_t *)data;
  i2c->xfer->num = num;
  i2c->xfer->cnt = 0U;
  i2c->xfer->addr = (uint16_t)(addr);
  i2c->xfer->ctrl = 0U;

  /* Set master mode */
  i2c->reg->TWIHS_CR = TWIHS_CR_MSEN | TWIHS_CR_SVDIS;
  
  /* Set slave address and number of internal address bytes. */
  i2c->reg->TWIHS_MMR = 0;
  if (addr & ARM_I2C_ADDRESS_10BIT) {
    i2c->reg->TWIHS_MMR = TWIHS_MMR_IADRSZ_1_BYTE;
    i2c->reg->TWIHS_MMR |= TWIHS_MMR_DADR(0xF0 | ((i2c->xfer->addr >> 8) & 0x3));
    i2c->reg->TWIHS_IADR = (i2c->xfer->addr & 0xFF);
  } else {
    i2c->reg->TWIHS_MMR = TWIHS_MMR_IADRSZ_NONE | (i2c->xfer->addr << 16);
  }

  if (xfer_pending) {
    i2c->xfer->ctrl |= XFER_CTRL_XPENDING;
  }
  
#if defined(I2C_MODE_IRQ)
  {
    i2c->reg->TWIHS_IER = TWIHS_IER_TXRDY;
  }
#else
  {
/* Send all bytes */
  while (i2c->xfer->num > 0) {
    status = i2c->reg->TWIHS_SR;
    if (status & TWIHS_SR_NACK) {
      event |= ARM_I2C_EVENT_ADDRESS_NACK;
      break;
    }

    if (!(status & TWIHS_SR_TXRDY)) {
      continue;
    }
    i2c->reg->TWIHS_THR = i2c->xfer->buf[i2c->xfer->cnt++];

    i2c->xfer->num--;
  }

  while (1) {
    status = i2c->reg->TWIHS_SR;
    if (status & TWIHS_SR_NACK) {
      event |= ARM_I2C_EVENT_ADDRESS_NACK;
      break;
    }

    if (status & TWIHS_SR_TXRDY) {
      break;
    }
  }

  if (!xfer_pending) {
    i2c->reg->TWIHS_CR = TWIHS_CR_STOP;
      while ((i2c->reg->TWIHS_SR & TWIHS_SR_TXCOMP) != TWIHS_SR_TXCOMP);
      i2c->xfer->ctrl &= ~XFER_CTRL_XPENDING;
    }           
      
    event |= ARM_I2C_EVENT_TRANSFER_DONE;

    i2c->info->cb_event(event);
      
    // Set RX busy flag
    i2c->info->status.busy = 0U;
  }
#endif

  return ARM_DRIVER_OK;
}

/**
   \fn          int32_t I2Cx_MasterReceive (uint32_t       addr,
   uint8_t       *data,
   uint32_t       num,
   bool           xfer_pending,
   I2C_RESOURCES *i2c)
   \brief       Start receiving data as I2C Master.
   \param[in]   addr          Slave address (7-bit or 10-bit)
   \param[out]  data          Pointer to buffer for data to receive from I2C Slave
   \param[in]   num           Number of data bytes to receive
   \param[in]   xfer_pending  Transfer operation is pending - Stop condition will not be generated
   \param[in]   i2c           Pointer to I2C resources
   \return      \ref execution_status
*/
static int32_t I2Cx_MasterReceive (uint32_t       addr,
                                   uint8_t       *data,
                                   uint32_t       num,
                                   bool           xfer_pending,
                                   I2C_RESOURCES *i2c) {
  uint32_t status = 0;
  uint32_t event = 0;        

  if ((data == NULL) || (num == 0U)) {
      return ARM_DRIVER_ERROR_PARAMETER;
  }
  if ((addr & ~(ARM_I2C_ADDRESS_10BIT | ARM_I2C_ADDRESS_GC)) > 0x3FFU) {
      return ARM_DRIVER_ERROR_PARAMETER;
  }

  if (i2c->info->status.busy) {
      return (ARM_DRIVER_ERROR_BUSY);
  }
  
  i2c->info->status.busy             = 1U;
  i2c->info->status.mode             = 1U;
  i2c->info->status.direction        = 1U;
  i2c->info->status.bus_error        = 0U;
  i2c->info->status.arbitration_lost = 0U;

  i2c->xfer->cnt  = 0U;
  i2c->xfer->num  = num;
  i2c->xfer->buf  = data;
  i2c->xfer->addr = (uint16_t)(addr);
  
  /* Set master mode */
  i2c->reg->TWIHS_CR = TWIHS_CR_MSEN | TWIHS_CR_SVDIS;
  
  /* Set internal address bytes */
  if (addr & ARM_I2C_ADDRESS_10BIT) {
    i2c->reg->TWIHS_MMR = TWIHS_MMR_MREAD | TWIHS_MMR_IADRSZ_1_BYTE | TWIHS_MMR_DADR(0xF0 | ((i2c->xfer->addr >> 8) & 0x7));
    i2c->reg->TWIHS_IADR = (i2c->xfer->addr & 0xFF);
  } else {
    i2c->reg->TWIHS_MMR =  TWIHS_MMR_IADRSZ_NONE | TWIHS_MMR_MREAD | (i2c->xfer->addr << 16);
  }

  if (xfer_pending) {
    i2c->xfer->ctrl |= XFER_CTRL_XPENDING;
  }
    
#if defined(I2C_MODE_IRQ)
  {
    i2c->reg->TWIHS_IER = TWIHS_IER_RXRDY;
    i2c->reg->TWIHS_CR = TWIHS_CR_START;
  }
#else
  {
    i2c->reg->TWIHS_CR = TWIHS_CR_START;
    while (i2c->xfer->num) {
      status = i2c->reg->TWIHS_SR;
      if (status & TWIHS_SR_NACK) {
        i2c->info->status.bus_error++;
        event |= ARM_I2C_EVENT_ADDRESS_NACK;
        break;
      }
      /* Last byte ? */
      if (i2c->xfer->num == 1) {
        i2c->reg->TWIHS_CR = TWIHS_CR_STOP;
      }
        
      if (!(status & TWIHS_SR_RXRDY)) {
        continue;
      }
      i2c->xfer->buf[i2c->xfer->cnt++] = i2c->reg->TWIHS_RHR;
        
      i2c->xfer->num--;
    }

    if (!xfer_pending) {
        while ((i2c->reg->TWIHS_SR & TWIHS_SR_TXCOMP) != TWIHS_SR_TXCOMP);
    }

    event |= ARM_I2C_EVENT_TRANSFER_DONE;

    i2c->xfer->ctrl &= ~XFER_CTRL_XPENDING;
    i2c->info->cb_event(event);
    // Set RX busy flag
    i2c->info->status.busy = 0U;
  }
#endif
    
    return ARM_DRIVER_OK;
}

/**
   \fn          int32_t I2Cx_SlaveTransmit (const uint8_t *data, uint32_t num, I2C_RESOURCES *i2c)
   \brief       Start transmitting data as I2C Slave.
   \param[in]   data          Pointer to buffer with data to send to I2C Master
   \param[in]   num           Number of data bytes to send
   \param[in]   i2c           Pointer to I2C resources
   \return      \ref execution_status
*/
static int32_t I2Cx_SlaveTransmit (const uint8_t *data, uint32_t num, I2C_RESOURCES *i2c)
{
  if ((data == NULL) || (num == 0U)) {
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if (i2c->info->status.busy) {
    return (ARM_DRIVER_ERROR_BUSY);
  }

  i2c->info->status.busy         = 1U;
  i2c->info->status.mode         = 0U;
  i2c->info->status.direction    = 1U;
  i2c->info->status.bus_error    = 0U;
  i2c->info->status.general_call = 0U;

  i2c->xfer->num  = num;
  i2c->xfer->cnt  = 0U;
  i2c->xfer->buf  = (uint8_t *)data;
  i2c->xfer->ctrl = 0U;

  i2c->reg->TWIHS_CR = TWIHS_CR_SVEN | TWIHS_CR_MSDIS;
  
#if defined(I2C_MODE_IRQ)
  {
    // Configure and enable TWIHS interrupt
    NVIC_ClearPendingIRQ(i2c->irq_num);
    NVIC_EnableIRQ(i2c->irq_num);

    i2c->reg->TWIHS_IER = TWIHS_IER_SVACC | TWIHS_IER_TXRDY;
  }
#else
  {
    uint32_t status;

    do {
      status = i2c->reg->TWIHS_SR;
      if (status & TWIHS_SR_SVACC) {
        if (!(status & (TWIHS_SR_GACC | TWIHS_SR_NACK)) &&
           ((status & (TWIHS_SR_SVREAD | TWIHS_SR_TXRDY))
           == (TWIHS_SR_SVREAD | TWIHS_SR_TXRDY))) {
          if (i2c->xfer->cnt < i2c->xfer->num)
            i2c->reg->TWIHS_THR = i2c->xfer->buf[i2c->xfer->cnt++];
        }
      } else if ((status & (TWIHS_SR_EOSACC | TWIHS_SR_TXCOMP))
                == (TWIHS_SR_EOSACC | TWIHS_SR_TXCOMP)) {
        break;
      }
    } while (1);

    i2c->info->cb_event(ARM_I2C_EVENT_TRANSFER_DONE);

    i2c->info->status.busy = 0U;
  }
#endif

  return ARM_DRIVER_OK;
}

/**
   \fn          int32_t I2Cx_SlaveReceive (uint8_t *data, uint32_t num, I2C_RESOURCES *i2c)
   \brief       Start receiving data as I2C Slave.
   \param[out]  data          Pointer to buffer for data to receive from I2C Master
   \param[in]   num           Number of data bytes to receive
   \param[in]   i2c           Pointer to I2C resources
   \return      \ref execution_status
*/
static int32_t I2Cx_SlaveReceive (uint8_t *data, uint32_t num, I2C_RESOURCES *i2c) {

  if ((data == NULL) || (num == 0U)) {
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if (i2c->info->status.busy) {
    return (ARM_DRIVER_ERROR_BUSY);
  }

  i2c->info->status.busy         = 1U;
  i2c->info->status.mode         = 0U;
  i2c->info->status.direction    = 0U;
  i2c->info->status.bus_error    = 0U;
  i2c->info->status.general_call = 0U;

  i2c->xfer->num  = num;
  i2c->xfer->cnt  = 0U;
  i2c->xfer->buf = data;
  i2c->xfer->ctrl = 0U;

  i2c->reg->TWIHS_CR = TWIHS_CR_SVEN | TWIHS_CR_MSDIS;
  
#if defined(I2C_MODE_IRQ)
  {
    // Configure and enable SPI interrupt
    NVIC_ClearPendingIRQ(i2c->irq_num);
    NVIC_EnableIRQ(i2c->irq_num);
    i2c->reg->TWIHS_IER = TWIHS_IER_SVACC | TWIHS_IER_EOSACC | TWIHS_IER_RXRDY;
  }
#else
  {
    uint32_t status;
      
    do {
      status = i2c->reg->TWIHS_SR;
      if (status & TWIHS_SR_SVACC) {
        if (!(status & (TWIHS_SR_GACC | TWIHS_SR_SVREAD)) &&
           (status & TWIHS_SR_RXRDY)) {
           i2c->xfer->buf[i2c->xfer->cnt++] = (uint8_t) i2c->reg->TWIHS_RHR;
        }
      } else if ((status & (TWIHS_SR_EOSACC | TWIHS_SR_TXCOMP))
                == (TWIHS_SR_EOSACC | TWIHS_SR_TXCOMP)) {
        break;
      }
    } while (1);
  
    i2c->info->cb_event(ARM_I2C_EVENT_TRANSFER_DONE);
  
    // Set RX busy flag
    i2c->info->status.busy = 0U;
  }   
#endif
  
  return ARM_DRIVER_OK;
}

static int32_t I2Cx_SetClock(uint32_t dwTwCk, uint32_t dwMCk, I2C_RESOURCES *i2c)
{
  uint32_t dwCkDiv = 0 ;
  uint32_t dwClDiv ;
  uint32_t dwOk = 0 ;
  
  /* Configure clock */
  while ( !dwOk ) {
    dwClDiv = ((dwMCk / (2 * dwTwCk)) - 4) / (1<<dwCkDiv) ;
    
    if ( dwClDiv <= 255 ) {
      dwOk = 1 ;
    } else {
      dwCkDiv++ ;
    }
  }
  
  if (dwCkDiv > 8)
    return ARM_DRIVER_ERROR_UNSUPPORTED;
  
  i2c->reg->TWIHS_CWGR = 0 ;
  i2c->reg->TWIHS_CWGR = (dwCkDiv << 16) | (dwClDiv << 8) | dwClDiv ;
  
  return ARM_DRIVER_OK;
}


/**
   \fn          int32_t I2Cx_GetDataCount (void)
   \brief       Get transferred data count.
   \return      number of data bytes transferred; -1 when Slave is not addressed by Master
*/
static int32_t I2Cx_GetDataCount (I2C_RESOURCES *i2c) {
  return ((int32_t)i2c->xfer->cnt);
}


/**
   \fn          int32_t I2Cx_Control (uint32_t control, uint32_t arg, I2C_RESOURCES *i2c)
   \brief       Control I2C Interface.
   \param[in]   control  operation
   \param[in]   arg      argument of operation (optional)
   \param[in]   i2c      pointer to I2C resources
   \return      \ref execution_status
*/
static int32_t I2Cx_Control(uint32_t control, uint32_t arg, I2C_RESOURCES *i2c) {
  if ((i2c->info->state & I2C_POWERED) == 0U) {
    /* I2C not powered */
    return ARM_DRIVER_ERROR;
  }

  switch (control) {
  case ARM_I2C_OWN_ADDRESS:
    if (arg & ARM_I2C_ADDRESS_10BIT) {
      return ARM_DRIVER_ERROR_UNSUPPORTED;
    } else {
      i2c->reg->TWIHS_SMR = TWIHS_SMR_SADR(arg & 0xff);
    }
    break;

  case ARM_I2C_BUS_SPEED:
    switch (arg) {
      case ARM_I2C_BUS_SPEED_STANDARD:
        return I2Cx_SetClock(100000,  SystemCoreClock, i2c);

      case ARM_I2C_BUS_SPEED_FAST:
        return I2Cx_SetClock(400000, SystemCoreClock, i2c);
      
      case ARM_I2C_BUS_SPEED_FAST_PLUS:
        return I2Cx_SetClock(1000000,  SystemCoreClock, i2c);

      case ARM_I2C_BUS_SPEED_HIGH:
        return I2Cx_SetClock(3400000, SystemCoreClock, i2c);      
    }
    return ARM_DRIVER_ERROR_UNSUPPORTED;

  case ARM_I2C_BUS_CLEAR:
    i2c->reg->TWIHS_CR = TWIHS_CR_CLEAR | TWIHS_CR_MSEN | TWIHS_CR_SVDIS;
    return ARM_DRIVER_OK;

  case ARM_I2C_ABORT_TRANSFER:
    /* Disable DMA requests and I2C interrupts */
    i2c->reg->TWIHS_IDR &= TWIHS_IDR_TXRDY | TWIHS_IDR_RXRDY | TWIHS_IDR_TXCOMP;
    

    if (i2c->dma_tx != NULL) {
      // Abort TX DMA transfer
      XDMAD_StopTransfer(i2c->dma, *i2c->dma_tx);
    }

    if (i2c->dma_rx != NULL) {
      // Abort TX DMA transfer
      XDMAD_StopTransfer(i2c->dma, *i2c->dma_rx);
    }
    /* Generate stop */
    /* Master generates stop after the current byte transfer */
    /* Slave releases SCL and SDA after the current byte transfer */
    i2c->reg->TWIHS_CR |= TWIHS_CR_STOP; // ABORT

    i2c->xfer->num  = 0U;
    i2c->xfer->cnt  = 0U;
    i2c->xfer->buf  = NULL;
    i2c->xfer->addr = 0U;
    i2c->xfer->ctrl = 0U;

    i2c->info->status.busy             = 0U;
    i2c->info->status.mode             = 0U;
    i2c->info->status.direction        = 0U;
    i2c->info->status.general_call     = 0U;
    i2c->info->status.arbitration_lost = 0U;
    i2c->info->status.bus_error        = 0U;

    /* Disable and reenable peripheral to clear some state */
    i2c->reg->TWIHS_CR = TWIHS_CR_SWRST;
    break;

    default: return ARM_DRIVER_ERROR;
  }
  
  return ARM_DRIVER_OK;
}


/**
   \fn          ARM_I2C_STATUS I2Cx_GetStatus (I2C_RESOURCES *i2c)
   \brief       Get I2C status.
   \param[in]   i2c      pointer to I2C resources
   \return      I2C status \ref ARM_I2C_STATUS
*/
static ARM_I2C_STATUS I2Cx_GetStatus (I2C_RESOURCES *i2c) {
  return i2c->info->status;
}

#if (RTE_I2C0 == 1)
/* I2C0 Driver wrapper functions */
static ARM_I2C_CAPABILITIES I2C0_GetCapabilities(void) {
  return I2Cx_GetCapabilities(&I2C0_Resources);
}
static int32_t I2C0_Initialize (ARM_I2C_SignalEvent_t cb_event) {
  return I2Cx_Initialize(cb_event, &I2C0_Resources);
}
static int32_t I2C0_Uninitialize (void) {
  return I2Cx_Uninitialize(&I2C0_Resources);
}
static int32_t I2C0_PowerControl (ARM_POWER_STATE state) {
  return I2Cx_PowerControl(state, &I2C0_Resources);
}
static int32_t I2C0_MasterTransmit (uint32_t addr, const uint8_t *data, uint32_t num, bool xfer_pending) {
  return I2Cx_MasterTransmit(addr, data, num, xfer_pending, &I2C0_Resources);
}
static int32_t I2C0_MasterReceive (uint32_t addr, uint8_t *data, uint32_t num, bool xfer_pending) {
  return I2Cx_MasterReceive(addr, data, num, xfer_pending, &I2C0_Resources);
}
static int32_t I2C0_SlaveTransmit (const uint8_t *data, uint32_t num) {
  return I2Cx_SlaveTransmit(data, num, &I2C0_Resources);
}
static int32_t I2C0_SlaveReceive (uint8_t *data, uint32_t num) {
  return I2Cx_SlaveReceive(data, num, &I2C0_Resources);
}
static int32_t I2C0_GetDataCount (void) {
  return I2Cx_GetDataCount(&I2C0_Resources);
}
static int32_t I2C0_Control (uint32_t control, uint32_t arg) {
  return I2Cx_Control(control, arg, &I2C0_Resources);
}
static ARM_I2C_STATUS I2C0_GetStatus (void) {
  return I2Cx_GetStatus(&I2C0_Resources);
}
void TWIHS0_Handler (void) {
  I2Cx_IRQHandler(&I2C0_Resources);
}

/* I2C0 Driver Control Block */
ARM_DRIVER_I2C Driver_I2C0 = {
  I2Cx_GetVersion,
  I2C0_GetCapabilities,
  I2C0_Initialize,
  I2C0_Uninitialize,
  I2C0_PowerControl,
  I2C0_MasterTransmit,
  I2C0_MasterReceive,
  I2C0_SlaveTransmit,
  I2C0_SlaveReceive,
  I2C0_GetDataCount,
  I2C0_Control,
  I2C0_GetStatus
};
#endif /* #if (RTE_I2C0 == 1) ... */

#if (RTE_I2C1 == 1)
/* I2C1 Driver wrapper functions */
static ARM_I2C_CAPABILITIES I2C1_GetCapabilities(void) {
  return I2Cx_GetCapabilities(&I2C1_Resources);
}
static int32_t I2C1_Initialize (ARM_I2C_SignalEvent_t cb_event) {
  return I2Cx_Initialize(cb_event, &I2C1_Resources);
}
static int32_t I2C1_Uninitialize (void) {
  return I2Cx_Uninitialize(&I2C1_Resources);
}
static int32_t I2C1_PowerControl (ARM_POWER_STATE state) {
  return I2Cx_PowerControl(state, &I2C1_Resources);
}
static int32_t I2C1_MasterTransmit (uint32_t addr, const uint8_t *data, uint32_t num, bool xfer_pending) {
  return I2Cx_MasterTransmit(addr, data, num, xfer_pending, &I2C1_Resources);
}
static int32_t I2C1_MasterReceive (uint32_t addr, uint8_t *data, uint32_t num, bool xfer_pending) {
  return I2Cx_MasterReceive(addr, data, num, xfer_pending, &I2C1_Resources);
}
static int32_t I2C1_SlaveTransmit (const uint8_t *data, uint32_t num) {
  return I2Cx_SlaveTransmit(data, num, &I2C1_Resources);
}
static int32_t I2C1_SlaveReceive (uint8_t *data, uint32_t num) {
  return I2Cx_SlaveReceive(data, num, &I2C1_Resources);
}
static int32_t I2C1_GetDataCount (void) {
  return I2Cx_GetDataCount(&I2C1_Resources);
}
static int32_t I2C1_Control (uint32_t control, uint32_t arg) {
  return I2Cx_Control(control, arg, &I2C1_Resources);
}
static ARM_I2C_STATUS I2C1_GetStatus (void) {
  return I2Cx_GetStatus(&I2C1_Resources);
}
void TWIHS1_Handler (void) {
  I2Cx_IRQHandler(&I2C1_Resources);
}

/* I2C1 Driver Control Block */
ARM_DRIVER_I2C Driver_I2C1 = {
  I2Cx_GetVersion,
  I2C1_GetCapabilities,
  I2C1_Initialize,
  I2C1_Uninitialize,
  I2C1_PowerControl,
  I2C1_MasterTransmit,
  I2C1_MasterReceive,
  I2C1_SlaveTransmit,
  I2C1_SlaveReceive,
  I2C1_GetDataCount,
  I2C1_Control,
  I2C1_GetStatus
};
#endif /* #if (RTE_I2C1 == 1) ... */

#if (RTE_I2C2 == 1)
/* I2C2 Driver wrapper functions */
static ARM_I2C_CAPABILITIES I2C2_GetCapabilities(void) {
  return I2Cx_GetCapabilities(&I2C2_Resources);
}
static int32_t I2C2_Initialize (ARM_I2C_SignalEvent_t cb_event) {
  return I2Cx_Initialize(cb_event, &I2C2_Resources);
}
static int32_t I2C2_Uninitialize (void) {
  return I2Cx_Uninitialize(&I2C2_Resources);
}
static int32_t I2C2_PowerControl (ARM_POWER_STATE state) {
  return I2Cx_PowerControl(state, &I2C2_Resources);
}
static int32_t I2C2_MasterTransmit (uint32_t addr, const uint8_t *data, uint32_t num, bool xfer_pending) {
  return I2Cx_MasterTransmit(addr, data, num, xfer_pending, &I2C2_Resources);
}
static int32_t I2C2_MasterReceive (uint32_t addr, uint8_t *data, uint32_t num, bool xfer_pending) {
  return I2Cx_MasterReceive(addr, data, num, xfer_pending, &I2C2_Resources);
}
static int32_t I2C2_SlaveTransmit (const uint8_t *data, uint32_t num) {
  return I2Cx_SlaveTransmit(data, num, &I2C2_Resources);
}
static int32_t I2C2_SlaveReceive (uint8_t *data, uint32_t num) {
  return I2Cx_SlaveReceive(data, num, &I2C2_Resources);
}
static int32_t I2C2_GetDataCount (void) {
  return I2Cx_GetDataCount(&I2C2_Resources);
}
static int32_t I2C2_Control (uint32_t control, uint32_t arg) {
  return I2Cx_Control(control, arg, &I2C2_Resources);
}
static ARM_I2C_STATUS I2C2_GetStatus (void) {
  return I2Cx_GetStatus(&I2C2_Resources);
}
void TWIHS2_Handler (void) {
  I2Cx_IRQHandler(&I2C2_Resources);
}

/* I2C2 Driver Control Block */
ARM_DRIVER_I2C Driver_I2C2 = {
  I2Cx_GetVersion,
  I2C2_GetCapabilities,
  I2C2_Initialize,
  I2C2_Uninitialize,
  I2C2_PowerControl,
  I2C2_MasterTransmit,
  I2C2_MasterReceive,
  I2C2_SlaveTransmit,
  I2C2_SlaveReceive,
  I2C2_GetDataCount,
  I2C2_Control,
  I2C2_GetStatus
};
#endif /* #if (RTE_I2C2 == 1) ... */
