/* -----------------------------------------------------------------------------
 * Copyright (c) 2015 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *   
 * $Date:        27. August 2015
 * $Revision:    V1.02
 *  
 * Project:      CMSIS I2C Driver for Atmel SAM-ESV7
 * --------------------------------------------------------------------------*/

#ifndef __I2C_SAM_ESV7_H
#define __I2C_SAM_ESV7_H

#include "RTE_Device.h"
#include "Driver_I2C.h"
#include "chip.h"
#include <string.h>

// I2C Input/Output Configuration
typedef struct _I2C_IO {
    Pin              *scl;           // Pointer to SCL pin configuration
    Pin              *sda;           // Pointer to SDA pin configuration
} I2C_IO;

// I2C Information (Run-time)
typedef struct _I2C_INFO {
    ARM_I2C_SignalEvent_t cb_event;       // Event Callback
    ARM_I2C_STATUS        status;         // Status flags
    uint8_t               state;          // Current I2C state
    uint32_t              mode;           // Current I2C mode
} I2C_INFO;

// I2C Transfer Information (Run-Time)
typedef struct _I2C_TRANSFER_INFO {
    uint32_t              num;                  // Number of data to transfer
    uint32_t              cnt;                  // Data transfer counter
    uint8_t              *buf;                  // Data pointer
    uint16_t              addr;                 // Device address
    uint8_t               ctrl;                 // Transfer control flags
} I2C_TRANSFER_INFO;

// I2C Resources definition
typedef struct {
    ARM_I2C_CAPABILITIES  capabilities;         // Driver capabilities
    Twihs    *reg;                  // HW register
    uint32_t              id;                   // Peripheral id
    IRQn_Type             irq_num;              // I2C IRQ Number
    I2C_IO               *io;                   // I2C Input/Output pins
    I2C_INFO             *info;                 // Run-Time Information
    I2C_TRANSFER_INFO    *xfer;                 // I2C transfer information
    sXdmad               *dma;                  // DMA driver instance
    uint32_t             *dma_tx;               // DMA TX channel
    uint32_t             *dma_rx;               // DMA RX channel
} I2C_RESOURCES;

// Current driver status flag definition
#define I2C_INITIALIZED           ((uint8_t)(1U))          // I2C initialized
#define I2C_POWERED               ((uint8_t)(1U << 1))     // I2C powered on
#define I2C_CONFIGURED            ((uint8_t)(1U << 2))     // I2C configured
#define I2C_DATA_LOST             ((uint8_t)(1U << 3))     // I2C data lost occurred
#define I2C_MODE_FAULT            ((uint8_t)(1U << 4))     // I2C mode fault occurred

/* Transfer status flags definitions */
#define XFER_CTRL_XPENDING  ((uint8_t)0x01)   // Transfer pending

#endif /* ! __I2C_SAM_ESV7_H */
