/* -----------------------------------------------------------------------------
 * Copyright (c) 2015 ARM Ltd.
 *  
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *  
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 * 
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * $Date:        30. April 2015
 * $Revision:    V1.02
 *
 * Project:      CMSIS SPI Driver for Atmel SAM-ESV7
 * -------------------------------------------------------------------------- */
 
#include "SPI_SAM-ESV7.h"

#define ARM_SPI_DRV_VERSION ARM_DRIVER_VERSION_MAJOR_MINOR(1,02)   /* driver version */

/* Driver Version */
static const ARM_DRIVER_VERSION DriverVersion = {
  ARM_SPI_API_VERSION,
  ARM_SPI_DRV_VERSION
};

/* Driver Capabilities */
static const ARM_SPI_CAPABILITIES DriverCapabilities =
{
  1,  // Simplex Mode (Master and Slave)
  0,  // TI Synchronous Serial Interface
  0,  // Microwire Interface
  0   // Signal Mode Fault event
};

#if ((RTE_SPI0!=0)&((RTE_SPI0_DMA_TX!=0)|(RTE_SPI0_DMA_RX!=0)))| \
    ((RTE_SPI1!=0)&((RTE_SPI1_DMA_TX!=0)|(RTE_SPI1_DMA_RX!=0)))
#define DMA_SPI_EN 1
#else
#define DMA_SPI_EN 0
#endif

#if (DMA_SPI_EN!=0)
static sXdmad SPIx_Dma;
#endif

/* SPI0 */
#if (RTE_SPI0 != 0)

// SPI0 Status
static SPI_INFO SPI0_Info = {0};
static SPI_TRANSFER_INFO SPI0_TransferInfo = {0};

// SPI0 Configure pins
#ifdef RTE_SPI0_MISO_PORT
static Pin SPI0_MISO_PORT = RTE_SPI0_MISO_PORT;
#endif  
#ifdef RTE_SPI0_MOSI_PORT    
static Pin SPI0_MOSI_PORT = RTE_SPI0_MOSI_PORT;
#endif
#ifdef RTE_SPI0_SPCK_PORT    
static Pin SPI0_SPCK_PORT = RTE_SPI0_SPCK_PORT;
#endif
#ifdef RTE_SPI0_NPCS_PORT    
static Pin SPI0_NPCS_PORT = RTE_SPI0_NPCS_PORT;
#endif

static SPI_IO SPI0_Pins = {
#ifdef RTE_SPI0_MISO_PORT
  &SPI0_MISO_PORT,
#else
  NULL,
#endif
#ifdef RTE_SPI0_MOSI_PORT
  &SPI0_MOSI_PORT,
#else
  NULL,
#endif
  #ifdef RTE_SPI0_SPCK_PORT
  &SPI0_SPCK_PORT,
#else
  NULL,
#endif
  #ifdef RTE_SPI0_NPCS_PORT
  &SPI0_NPCS_PORT,
#else
  NULL,
#endif
};

#if (RTE_SPI0_DMA_TX!=0)
uint32_t SPI0_DmaTxChannel;
#endif
#if (RTE_SPI0_DMA_RX!=0)
uint32_t SPI0_DmaRxChannel;
#endif

// SPI0 Resources
static const SPI_RESOURCES SPI0_Resources = {
  SPI0,
  ID_SPI0,
  SPI0_IRQn,
  RTE_SPI0_NPCS_NUM,
  &SPI0_Pins,
  &SPI0_Info,
  &SPI0_TransferInfo,
#if (RTE_SPI0_DMA_TX!=0)|(RTE_SPI0_DMA_RX!=0)
  &SPIx_Dma,
#else
  NULL,
#endif
#if (RTE_SPI0_DMA_TX!=0)
  &SPI0_DmaTxChannel,
#else
  NULL,
#endif
#if (RTE_SPI0_DMA_RX!=0)
  &SPI0_DmaRxChannel,
#else
  NULL,
#endif
};

#endif

/* SPI1 */
#if (RTE_SPI1 != 0)

// SPI1 Status
static SPI_INFO SPI1_Info = {0};
static SPI_TRANSFER_INFO SPI1_TransferInfo = {0};

// SPI1 Configure pins
#ifdef RTE_SPI1_MISO_PORT
static Pin SPI1_MISO_PORT = RTE_SPI1_MISO_PORT;
#endif  
#ifdef RTE_SPI1_MOSI_PORT    
static Pin SPI1_MOSI_PORT = RTE_SPI1_MOSI_PORT;
#endif
#ifdef RTE_SPI1_SPCK_PORT    
static Pin SPI1_SPCK_PORT = RTE_SPI1_SPCK_PORT;
#endif
#ifdef RTE_SPI1_NPCS_PORT    
static Pin SPI1_NPCS_PORT = RTE_SPI1_NPCS_PORT;
#endif

static SPI_IO SPI1_Pins = {
#ifdef RTE_SPI1_MISO_PORT
  &SPI1_MISO_PORT,
#else
  NULL,
#endif
#ifdef RTE_SPI1_MOSI_PORT
  &SPI1_MOSI_PORT,
#else
  NULL,
#endif
  #ifdef RTE_SPI1_SPCK_PORT
  &SPI1_SPCK_PORT,
#else
  NULL,
#endif
  #ifdef RTE_SPI1_NPCS_PORT
  &SPI1_NPCS_PORT,
#else
  NULL,
#endif
};

#if (RTE_SPI1_DMA_TX!=0)
uint32_t SPI1_DmaTxChannel;
#endif
#if (RTE_SPI1_DMA_RX!=0)
uint32_t SPI1_DmaRxChannel;
#endif

// SPI1 Resources
static const SPI_RESOURCES SPI1_Resources = {
  SPI1,
  ID_SPI1,
  SPI1_IRQn,
  RTE_SPI1_NPCS_NUM,
  &SPI1_Pins,
  &SPI1_Info,
  &SPI1_TransferInfo,
#if (RTE_SPI1_DMA_TX!=0)|(RTE_SPI1_DMA_RX!=0)
  &SPIx_Dma,
#else
  NULL,
#endif
#if (RTE_SPI1_DMA_TX!=0)
  &SPI1_DmaTxChannel,
#else
  NULL,
#endif
#if (RTE_SPI1_DMA_RX!=0)
  &SPI1_DmaRxChannel,
#else
  NULL,
#endif

};

#endif


#if (DMA_SPI_EN!=0)
/*SPIx DMA Callback */
static void SPIx_DMA_Callback(uint32_t channel, const SPI_RESOURCES *spi)
{
  uint32_t event = 0;
  
  if (channel != *spi->dma_tx) return;

  // Configure and enable interrupt on RC compare  
  NVIC_ClearPendingIRQ(XDMAC_IRQn);
  NVIC_DisableIRQ(XDMAC_IRQn);
  
  // Update SPI info
  spi->reg->SPI_CR = SPI_CR_SPIDIS;
  spi->xfer->rx_cnt = spi->xfer->num;
  spi->xfer->num = 0U;
  spi->info->status.busy = 0U;
  event |= ARM_SPI_EVENT_TRANSFER_COMPLETE;
  if (spi->info->cb_event!=NULL) spi->info->cb_event(event);
  
  SCB_CleanInvalidateDCache();
}
#endif

/*SPI IRQ Handler */
void SPIx_IRQHandler (const SPI_RESOURCES *spi) {
  uint32_t event = 0;
  uint32_t status = spi->reg->SPI_SR;
  uint16_t tdr, rdr;

  // Error
  if ((status & (SPI_SR_OVRES | SPI_SR_UNDES)) != 0 ) { 
    NVIC_ClearPendingIRQ(spi->irq_num);
    NVIC_DisableIRQ(spi->irq_num);
    spi->reg->SPI_IDR = SPI_IDR_RDRF | SPI_IDR_TXEMPTY;   
    spi->reg->SPI_CR = SPI_CR_SPIDIS;
    spi->xfer->num = 0U;
    spi->info->status.busy = 0U;
    event |= ARM_SPI_EVENT_DATA_LOST;
    if (spi->info->cb_event!=NULL) spi->info->cb_event(event);
  }
  
  // RX
  if ((status & SPI_SR_RDRF) != 0 ) {
    // Read RDR register
    rdr = spi->reg->SPI_RDR & 0xFFFF; 
    if (spi->xfer->rx_buf != NULL) { 
      *spi->xfer->rx_buf++ = rdr;
      if ((spi->reg->SPI_CSR[spi->npcs_num]&SPI_CSR_BITS_Msk)!=0)
        *spi->xfer->rx_buf++ = rdr >> 8U;      
    } 
    spi->xfer->rx_cnt++;   
    
    // Transfer is completed
    if (spi->xfer->rx_cnt >= spi->xfer->num) {
      NVIC_ClearPendingIRQ(spi->irq_num);
      NVIC_DisableIRQ(spi->irq_num);
      spi->reg->SPI_IDR = SPI_IDR_RDRF | SPI_IDR_TXEMPTY;     
      spi->reg->SPI_CR = SPI_CR_SPIDIS;
      spi->xfer->num = 0U;
      spi->info->status.busy = 0U;
      event |= ARM_SPI_EVENT_TRANSFER_COMPLETE;
      if (spi->info->cb_event!=NULL) spi->info->cb_event(event);
    } 
  } 
  
  // TX
  if ((status & SPI_SR_TXEMPTY) != 0 ) {
    // Write TDR register  
    if (spi->xfer->tx_buf != NULL) {
      tdr = *spi->xfer->tx_buf++ | SPI_PCS(spi->npcs_num);   
      if ((spi->reg->SPI_CSR[spi->npcs_num]&SPI_CSR_BITS_Msk)!=0)
        tdr |= *spi->xfer->tx_buf++ << 8U;          
    } else {
      tdr = spi->xfer->def_val | SPI_PCS(spi->npcs_num); 
    }
    spi->reg->SPI_TDR = tdr;
    spi->xfer->tx_cnt++;
  }
}

/**
  \fn          ARM_DRIVER_VERSION SPIx_GetVersion (void)
  \brief       Get SPI driver version.
  \return      \ref ARM_DRV_VERSION
*/
static ARM_DRIVER_VERSION SPIx_GetVersion (void) {
  return DriverVersion;
}

/**
  \fn          ARM_SPIx_CAPABILITIES SPI_GetCapabilities (void)
  \brief       Get driver capabilities.
  \return      \ref ARM_SPI_CAPABILITIES
*/
static ARM_SPI_CAPABILITIES SPIx_GetCapabilities (void) {
  return DriverCapabilities;
}

/**
  \fn          int32_t SPIx_Initialize (ARM_SPI_SignalEvent_t cb_event, const SPI_RESOURCES *spi)
  \brief       Initialize SPI Interface.
  \param[in]   cb_event  Pointer to \ref ARM_SPI_SignalEvent
  \param[in]   spi       Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPIx_Initialize (ARM_SPI_SignalEvent_t cb_event, const SPI_RESOURCES *spi) {
  
  if (spi->info->state & SPI_INITIALIZED) { return ARM_DRIVER_OK; }
  if (spi->info->state & SPI_POWERED)     { return ARM_DRIVER_ERROR; }
    
  // Initialize SPI Run-Time Resources
  spi->info->cb_event = cb_event;
  spi->info->status.busy       = 0U;
  spi->info->status.data_lost  = 0U;
  spi->info->status.mode_fault = 0U;
  
  // Clear transfer information
  memset(spi->xfer, 0, sizeof(SPI_TRANSFER_INFO));
  
  // Pin configuration
  PIO_Configure(spi->io->miso,1);
  PIO_Configure(spi->io->mosi,1);
  PIO_Configure(spi->io->spck,1);
      
  spi->info->state = SPI_INITIALIZED;
  
  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t SPIx_Uninitialize (SPI_RESOURCES *spi)
  \brief       De-initialize SPI Interface.
  \param[in]   spi  Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPIx_Uninitialize (const SPI_RESOURCES *spi) {

  spi->info->state = 0;                 // SPI is uninitialized

  return ARM_DRIVER_OK;
}


/**
  \fn          int32_t SPI_PowerControl (ARM_POWER_STATE state, const SPI_RESOURCES *spi)
  \brief       Control SPI Interface Power.
  \param[in]   state  Power state
  \param[in]   spi    Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPIx_PowerControl (ARM_POWER_STATE state, const SPI_RESOURCES *spi) {

  switch (state) {
    
    case ARM_POWER_OFF:

#if (DMA_SPI_EN!=0) 
      if ((spi->dma_tx != NULL)||(spi->dma_rx != NULL)) {
        // Disable DMA Interrupts
        NVIC_ClearPendingIRQ(XDMAC_IRQn);
        NVIC_DisableIRQ(XDMAC_IRQn);
        
        if (spi->dma_tx != NULL) {
          XDMAD_FreeChannel(spi->dma, *spi->dma_tx);
        }
        if (spi->dma_rx != NULL) {
          XDMAD_FreeChannel(spi->dma, *spi->dma_rx);
        }       
      } 
      else 
#endif
      {
        // Disable SPI Interrupts
        NVIC_ClearPendingIRQ(spi->irq_num);
        NVIC_DisableIRQ(spi->irq_num);
        spi->reg->SPI_IDR = SPI_IDR_RDRF | SPI_IDR_TXEMPTY;
      }

      // Disable SPI peripheral
      spi->reg->SPI_CR = SPI_CR_SPIDIS;
      PMC_DisablePeripheral(spi->id);

      // Clear powered flag
      spi->info->state &= ~ SPI_POWERED;
      break;

    case ARM_POWER_FULL:
      // Check if SPI already powered
      if (spi->info->state & SPI_POWERED) { return ARM_DRIVER_OK; }
      if ((spi->info->state & SPI_INITIALIZED) == 0U) { return ARM_DRIVER_ERROR; }
      if ( spi->info->status.busy)                    { return ARM_DRIVER_ERROR_BUSY; }

      // Enable peripheral's clock
      PMC_EnablePeripheral(spi->id);
      // Disable SPI
      spi->reg->SPI_CR = SPI_CR_SPIDIS;
      // Execute a software reset of the SPI twice
      spi->reg->SPI_CR = SPI_CR_SWRST;
      spi->reg->SPI_CR = SPI_CR_SWRST;

#if (DMA_SPI_EN!=0) 
      if ((spi->dma_tx != NULL)||(spi->dma_rx != NULL)) {
        // DMA Driver initialize 
        XDMAD_Initialize(spi->dma, 0);
      }
      
      if (spi->dma_tx != NULL) {
        XDMAD_FreeChannel(spi->dma, *spi->dma_tx);  
        // Allocate a DMA channel for SPI0/1 TX
        *spi->dma_tx = XDMAD_AllocateChannel(spi->dma, XDMAD_TRANSFER_MEMORY, spi->id);
        if (*spi->dma_tx == XDMAD_ALLOC_FAILED) return ARM_DRIVER_ERROR;
        // Setup callbacks for SPI0/1 TX 
        XDMAD_SetCallback(spi->dma, *spi->dma_tx, (XdmadTransferCallback)SPIx_DMA_Callback,(void*)spi); 
        if (XDMAD_PrepareChannel(spi->dma, *spi->dma_tx)) return ARM_DRIVER_ERROR;
      }

      if (spi->dma_rx != NULL) {
        XDMAD_FreeChannel(spi->dma, *spi->dma_rx);
        // Allocate a DMA channel for SPI0/1 RX
        *spi->dma_rx = XDMAD_AllocateChannel(spi->dma, spi->id, XDMAD_TRANSFER_MEMORY);
        if (*spi->dma_rx == XDMAD_ALLOC_FAILED) return ARM_DRIVER_ERROR;
        // Setup callbacks for SPI0/1 RX  
        XDMAD_SetCallback(spi->dma, *spi->dma_rx, NULL, NULL);
        if (XDMAD_PrepareChannel(spi->dma, *spi->dma_rx)) return ARM_DRIVER_ERROR;
      }     
#endif

      // Ready for operation - set powered flag
      spi->info->state |= SPI_POWERED;
      break;

    case ARM_POWER_LOW:
    default: return ARM_DRIVER_ERROR_UNSUPPORTED;
  }
  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t SPIx_Control (uint32_t control, uint32_t arg, const SPI_RESOURCES *spi)
  \brief       Control SPI Interface.
  \param[in]   control  operation
  \param[in]   arg      argument of operation (optional)
  \param[in]   spi      Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPIx_Control (uint32_t control, uint32_t arg, const SPI_RESOURCES *spi) {
  uint32_t mode = 0U;
  uint32_t val  = 0U;
  
  if ((spi->info->state & SPI_POWERED) == 0U) { return ARM_DRIVER_ERROR; }

  if ((control & ARM_SPI_CONTROL_Msk) == ARM_SPI_ABORT_TRANSFER) {

#if (DMA_SPI_EN!=0) 
    // Abort DMA transfers
    if ((spi->dma_tx != NULL)||(spi->dma_rx != NULL)) {
      XDMAD_StopTransfer(spi->dma,*spi->dma_tx);
    }
    if ((spi->dma_tx != NULL)||(spi->dma_rx != NULL)) {
      XDMAD_StopTransfer(spi->dma,*spi->dma_rx);
    }   
#endif
    
    // Disable SPI 
    spi->reg->SPI_CR = SPI_CR_SPIDIS;
    
    memset(spi->xfer, 0, sizeof(SPI_TRANSFER_INFO));
    spi->info->status.busy = 0U;
    return ARM_DRIVER_OK;
  }

  // Check for busy flag
  if (spi->info->status.busy) { return ARM_DRIVER_ERROR_BUSY; }

  switch (control & ARM_SPI_CONTROL_Msk) {
    case ARM_SPI_MODE_INACTIVE:
      mode |= ARM_SPI_MODE_INACTIVE;
      break;

    case ARM_SPI_MODE_MASTER:
      mode |= ARM_SPI_MODE_MASTER;
      spi->reg->SPI_MR |= SPI_MR_MSTR | SPI_MR_MODFDIS;
      break;

    case ARM_SPI_MODE_SLAVE:
      mode |= ARM_SPI_MODE_SLAVE;
      spi->reg->SPI_MR &= ~SPI_MR_MSTR;
      break;

    case ARM_SPI_MODE_MASTER_SIMPLEX:
      return ARM_SPI_ERROR_MODE;

    case ARM_SPI_MODE_SLAVE_SIMPLEX:
      return ARM_SPI_ERROR_MODE;

    case ARM_SPI_SET_BUS_SPEED:
      goto set_speed;

    case ARM_SPI_GET_BUS_SPEED:   
      // Calculate master clock
      val = SystemCoreClock;
      switch (PMC->PMC_MCKR & PMC_MCKR_MDIV_Msk) {
        case PMC_MCKR_MDIV_PCK_DIV2: val /= 2; break;
        case PMC_MCKR_MDIV_PCK_DIV4: val /= 4; break;
        case PMC_MCKR_MDIV_PCK_DIV3: val /= 3; break;
      }
      // Get baud rate register
      return val/((spi->reg->SPI_CSR[spi->npcs_num] & SPI_CSR_SCBR_Msk) >> SPI_CSR_SCBR_Pos);

    case ARM_SPI_SET_DEFAULT_TX_VALUE:
      spi->xfer->def_val = (uint16_t)(arg & 0xFFFFU);
      return ARM_DRIVER_OK;

    case ARM_SPI_CONTROL_SS:
      val = (spi->info->mode & ARM_SPI_CONTROL_Msk); 
      // Master modes
      if (val == ARM_SPI_MODE_MASTER) {
        val = spi->info->mode & ARM_SPI_SS_MASTER_MODE_Msk;
        // Check if NPCS pin is available and software slave select master is selected
        if ((spi->io->npcs != NULL) && (val == ARM_SPI_SS_MASTER_SW)) {
          // Set/Clear NPCS pin
          if (arg == ARM_SPI_SS_INACTIVE)
            PIO_Set(spi->io->npcs);
          else
            PIO_Clear(spi->io->npcs);
        } else return ARM_DRIVER_ERROR;
        return ARM_DRIVER_OK;
      }

    default:
      return ARM_DRIVER_ERROR_UNSUPPORTED;
  }

  // Frame format
  switch (control & ARM_SPI_FRAME_FORMAT_Msk) {
    case ARM_SPI_CPOL0_CPHA0:
      spi->reg->SPI_CSR[spi->npcs_num] &= ~SPI_CSR_CPOL;
      spi->reg->SPI_CSR[spi->npcs_num] &= ~SPI_CSR_NCPHA;
      break;
    case ARM_SPI_CPOL0_CPHA1:
      spi->reg->SPI_CSR[spi->npcs_num] &= ~SPI_CSR_CPOL;
      spi->reg->SPI_CSR[spi->npcs_num] |= SPI_CSR_NCPHA;
      break;
    case ARM_SPI_CPOL1_CPHA0:
      spi->reg->SPI_CSR[spi->npcs_num] |= SPI_CSR_CPOL;
      spi->reg->SPI_CSR[spi->npcs_num] &= ~SPI_CSR_NCPHA;
      break;
    case ARM_SPI_CPOL1_CPHA1:
      spi->reg->SPI_CSR[spi->npcs_num] |= SPI_CSR_CPOL;
      spi->reg->SPI_CSR[spi->npcs_num] |= SPI_CSR_NCPHA;
      break;
    case ARM_SPI_TI_SSI:
    case ARM_SPI_MICROWIRE:
      return ARM_SPI_ERROR_FRAME_FORMAT;
    default: return ARM_SPI_ERROR_FRAME_FORMAT;
  }

  // Data bits
  val = (control & ARM_SPI_DATA_BITS_Msk)>>ARM_SPI_DATA_BITS_Pos;
  if (val>=8U && val<=16U) {
    spi->reg->SPI_CSR[spi->npcs_num] &= ~SPI_CSR_BITS_Msk;
    spi->reg->SPI_CSR[spi->npcs_num] |= SPI_CSR_BITS(val-8U);
  } else { return ARM_SPI_ERROR_DATA_BITS; }

  // Bit order
  if ((control & ARM_SPI_BIT_ORDER_Msk) != ARM_SPI_MSB_LSB) { return ARM_SPI_ERROR_BIT_ORDER; }

  // Slave select master modes
  if (mode == ARM_SPI_MODE_MASTER) {
    switch (control & ARM_SPI_SS_MASTER_MODE_Msk) { 
      case ARM_SPI_SS_MASTER_UNUSED:
        if (spi->io->npcs != NULL) {
          // Configure NPCS pin as GPIO output
          val = spi->io->npcs->type;
          spi->io->npcs->type = PIO_OUTPUT_1;         
          PIO_Configure(spi->io->npcs,1);
          spi->io->npcs->type = val;
          mode |= ARM_SPI_SS_MASTER_UNUSED;
        }
        break;

      case ARM_SPI_SS_MASTER_SW:
        if (spi->io->npcs != NULL) {
          // Configure NPCS pin as GPIO output
          val = spi->io->npcs->type;
          spi->io->npcs->type = PIO_OUTPUT_1;         
          PIO_Configure(spi->io->npcs,1);
          spi->io->npcs->type = val;  
          mode |= ARM_SPI_SS_MASTER_SW;
        } else {
          // NPCS pin is not available
          return ARM_SPI_ERROR_SS_MODE;
        }
        break;

      case ARM_SPI_SS_MASTER_HW_INPUT:
          return ARM_SPI_ERROR_SS_MODE;
        
      case ARM_SPI_SS_MASTER_HW_OUTPUT:
        if (spi->io->npcs != NULL) {
          // Configure NPCS pin
          PIO_Configure(spi->io->npcs,1);
          spi->reg->SPI_MR |= SPI_MR_PCS_Msk;
          spi->reg->SPI_MR &= ~(SPI_MR_PCS(1<<spi->npcs_num));
          mode |= ARM_SPI_SS_MASTER_HW_OUTPUT;
        } else {
          // NPCS pin is not available
          return ARM_SPI_ERROR_SS_MODE;
        }
        break;
        default: return ARM_SPI_ERROR_SS_MODE;
    }
  }

  // Slave select slave modes
  if (spi->info->mode ==  ARM_SPI_MODE_SLAVE) {
    switch (control & ARM_SPI_SS_SLAVE_MODE_Msk) {
      case ARM_SPI_SS_SLAVE_HW:
        if ((spi->io->npcs != NULL)&&(spi->npcs_num != 0)) {
          // Configure NSS pin
          spi->reg->SPI_MR |= SPI_MR_PCS_Msk;
          spi->reg->SPI_MR &= ~(SPI_MR_PCS(0));
          mode |= ARM_SPI_SS_SLAVE_HW;
        } else {
          // NSS slave mode is only available on NPCS0
          return ARM_SPI_ERROR_SS_MODE;
        }
        break;

      case ARM_SPI_SS_SLAVE_SW:
        // SW Slave Select is not supported
      
      default: return ARM_SPI_ERROR_SS_MODE;
    }
  }

  // Save mode and state
  spi->info->mode = mode;

  if ((mode & ARM_SPI_CONTROL_Msk) == ARM_SPI_MODE_INACTIVE) {
    spi->info->state &= ~SPI_CONFIGURED;
  } else {
    spi->info->state |=  SPI_CONFIGURED;
  }

set_speed:
  // Calculate master clock
  val = SystemCoreClock;
  switch (PMC->PMC_MCKR & PMC_MCKR_MDIV_Msk) {
    case PMC_MCKR_MDIV_PCK_DIV2: val /= 2; break;
    case PMC_MCKR_MDIV_PCK_DIV4: val /= 4; break;
    case PMC_MCKR_MDIV_PCK_DIV3: val /= 3; break;
  }
  // Set baud rate register
  spi->reg->SPI_CSR[spi->npcs_num] &= ~SPI_CSR_SCBR_Msk;
  spi->reg->SPI_CSR[spi->npcs_num] |= SPI_SCBR(arg, val);
  
  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t SPIx_Transfer (const void *data_out, void *data_in, uint32_t num, const SPI_RESOURCES *spi)
  \brief       Start sending/receiving data to/from SPI transmitter/receiver.
  \param[in]   data_out  Pointer to buffer with data to send to SPI transmitter
  \param[out]  data_in   Pointer to buffer for data to receive from SPI receiver
  \param[in]   num       Number of data items to transfer
  \param[in]   spi       Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPIx_Transfer (const void *data_out, void *data_in, uint32_t num, const SPI_RESOURCES *spi) {
#if (DMA_SPI_EN!=0)
  sXdmadCfg xdmadTxCfg, xdmadRxCfg;
#endif
  
  if ((data_out == NULL) || (data_in == NULL) || (num == 0U)) { return ARM_DRIVER_ERROR_PARAMETER; }
  if ((spi->info->state & SPI_CONFIGURED) == 0U)              { return ARM_DRIVER_ERROR; }
  if ( spi->info->status.busy)                                { return ARM_DRIVER_ERROR_BUSY; }

  // Check if receive and transmit pins available
  if ((spi->io->miso == NULL) || (spi->io->mosi == NULL)) {
    return ARM_DRIVER_ERROR;
  }

  // Update SPI statuses
  spi->info->status.busy       = 1U;
  spi->info->status.data_lost  = 0U;
  spi->info->status.mode_fault = 0U;

  // Save transfer info
  spi->xfer->rx_buf = (uint8_t *)data_in;
  spi->xfer->tx_buf = (uint8_t *)data_out;
  spi->xfer->num    = num;
  spi->xfer->rx_cnt = 0U;
  spi->xfer->tx_cnt = 0U;
  
  // SPI enable
  spi->reg->SPI_CR = SPI_CR_SPIEN;
    
#if (DMA_SPI_EN!=0)     
  if ((spi->dma_tx != NULL)||(spi->dma_rx != NULL)) {
      
    // Configure and enable DMA interrupt   
    NVIC_ClearPendingIRQ(XDMAC_IRQn);
    NVIC_SetPriority( XDMAC_IRQn ,1);
    NVIC_EnableIRQ(XDMAC_IRQn);
  
    // Clean cache
    SCB_CleanInvalidateDCache();
    
    if (spi->dma_rx != NULL) {
      
      // Setup RX Link List
      xdmadRxCfg.mbr_sa = (uint32_t) &spi->reg->SPI_RDR;
      xdmadRxCfg.mbr_da = (uint32_t) spi->xfer->rx_buf;
      xdmadRxCfg.mbr_ubc = XDMA_UBC_NVIEW_NDV0 |
          XDMA_UBC_NDE_FETCH_DIS|
          XDMA_UBC_NDEN_UPDATED | spi->xfer->num;
      xdmadRxCfg.mbr_cfg = XDMAC_CC_TYPE_PER_TRAN |
          XDMAC_CC_MBSIZE_SINGLE |
          XDMAC_CC_DSYNC_PER2MEM |
          XDMAC_CC_CSIZE_CHK_1 |
          (((spi->reg->SPI_CSR[spi->npcs_num]&SPI_CSR_BITS_Msk)!=0) ? XDMAC_CC_DWIDTH_HALFWORD : XDMAC_CC_DWIDTH_BYTE) |
          XDMAC_CC_SIF_AHB_IF1 |
          XDMAC_CC_DIF_AHB_IF0 |
          XDMAC_CC_SAM_FIXED_AM |
          XDMAC_CC_DAM_INCREMENTED_AM |
          XDMAC_CC_PERID(XDMAIF_Get_ChannelNumber(spi->id, XDMAD_TRANSFER_RX));
      xdmadRxCfg.mbr_bc = 0;
      xdmadRxCfg.mbr_sus = 0;
      xdmadRxCfg.mbr_dus = 0;
      if (XDMAD_ConfigureTransfer(spi->dma, *spi->dma_rx, &xdmadRxCfg, 0, 0, XDMA_INT))
        return ARM_DRIVER_ERROR;
      if (XDMAD_StartTransfer(spi->dma, *spi->dma_rx)) 
        return ARM_DRIVER_ERROR;
    }     
    
    if (spi->dma_tx != NULL) {
              
      // Setup TX 
      xdmadTxCfg.mbr_sa = (uint32_t) spi->xfer->tx_buf;   
      xdmadTxCfg.mbr_da = (uint32_t) &spi->reg->SPI_TDR;
      xdmadTxCfg.mbr_ubc =  XDMA_UBC_NVIEW_NDV0 |
          XDMA_UBC_NDE_FETCH_DIS|
          XDMA_UBC_NSEN_UPDATED | spi->xfer->num;
      xdmadTxCfg.mbr_cfg = XDMAC_CC_TYPE_PER_TRAN |
          XDMAC_CC_MBSIZE_SINGLE |
          XDMAC_CC_DSYNC_MEM2PER |
          XDMAC_CC_CSIZE_CHK_1 |
          (((spi->reg->SPI_CSR[spi->npcs_num]&SPI_CSR_BITS_Msk)!=0) ? XDMAC_CC_DWIDTH_HALFWORD : XDMAC_CC_DWIDTH_BYTE) |
          XDMAC_CC_SIF_AHB_IF0 |
          XDMAC_CC_DIF_AHB_IF1 |
          XDMAC_CC_SAM_INCREMENTED_AM |
          XDMAC_CC_DAM_FIXED_AM |
          XDMAC_CC_PERID(XDMAIF_Get_ChannelNumber(spi->id, XDMAD_TRANSFER_TX));
      xdmadTxCfg.mbr_bc = 0;
      xdmadTxCfg.mbr_sus = 0;
      xdmadTxCfg.mbr_dus = 0;
      if (XDMAD_ConfigureTransfer(spi->dma, *spi->dma_tx, &xdmadTxCfg, 0, 0, XDMA_INT))
        return ARM_DRIVER_ERROR;
      if (XDMAD_StartTransfer(spi->dma, *spi->dma_tx)) 
        return ARM_DRIVER_ERROR;
    }   
  } 
  else
#endif  
  {   
    // Configure and enable SPI interrupt
    NVIC_ClearPendingIRQ(spi->irq_num);
    NVIC_SetPriority(spi->irq_num,1);
    NVIC_EnableIRQ(spi->irq_num);     
    spi->reg->SPI_IER = SPI_IER_RDRF | SPI_IER_TXEMPTY;     
  }
  
  return ARM_DRIVER_OK;
}


/**
  \fn          int32_t SPIx_Send (const void *data, uint32_t num, const SPI_RESOURCES *spi)
  \brief       Start sending data to SPI transmitter.
  \param[in]   data  Pointer to buffer with data to send to SPI transmitter
  \param[in]   num   Number of data items to send
  \param[in]   spi   Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPIx_Send (const void *data, uint32_t num, const SPI_RESOURCES *spi) {
#if (DMA_SPI_EN!=0) 
  sXdmadCfg xdmadTxCfg, xdmadRxCfg;
  uint16_t val;
#endif
  
  if ((data == NULL) || (num == 0U))             { return ARM_DRIVER_ERROR_PARAMETER; }
  if ((spi->info->state & SPI_CONFIGURED) == 0U) { return ARM_DRIVER_ERROR; }
  if ( spi->info->status.busy)                   { return ARM_DRIVER_ERROR_BUSY; }

  // Check if transmit pin available
  if ((((spi->io->mosi != NULL) && ((spi->info->mode & ARM_SPI_CONTROL_Msk) == ARM_SPI_MODE_MASTER)) ||
       ((spi->io->miso != NULL) && ((spi->info->mode & ARM_SPI_CONTROL_Msk) == ARM_SPI_MODE_SLAVE ))) == 0U) {
    return ARM_DRIVER_ERROR;
  }

  // Update SPI statuses
  spi->info->status.busy       = 1U;
  spi->info->status.data_lost  = 0U;
  spi->info->status.mode_fault = 0U;

  // Save transfer info
  spi->xfer->rx_buf = NULL;
  spi->xfer->tx_buf = (uint8_t *)data;
  spi->xfer->num    = num;
  spi->xfer->rx_cnt = 0U;
  spi->xfer->tx_cnt = 0U;

  // SPI enable
  spi->reg->SPI_CR = SPI_CR_SPIEN;
  
#if (DMA_SPI_EN!=0) 
  if ((spi->dma_tx != NULL)||(spi->dma_rx != NULL)) { 
    
    // Configure and enable DMA interrupt   
    NVIC_ClearPendingIRQ(XDMAC_IRQn);
    NVIC_SetPriority( XDMAC_IRQn ,1);
    NVIC_EnableIRQ(XDMAC_IRQn);
    
    // Clean cache
    SCB_CleanInvalidateDCache();
    
    if (spi->dma_rx != NULL) {
      // Setup RX Link List
      xdmadRxCfg.mbr_sa = (uint32_t) &spi->reg->SPI_RDR;
      xdmadRxCfg.mbr_da = (uint32_t) &val;
      xdmadRxCfg.mbr_ubc = XDMA_UBC_NVIEW_NDV0 |
          XDMA_UBC_NDE_FETCH_DIS|
          XDMA_UBC_NDEN_UPDATED | spi->xfer->num;
      xdmadRxCfg.mbr_cfg = XDMAC_CC_TYPE_PER_TRAN |
          XDMAC_CC_MBSIZE_SINGLE |
          XDMAC_CC_DSYNC_PER2MEM |
          XDMAC_CC_CSIZE_CHK_1 |
          (((spi->reg->SPI_CSR[spi->npcs_num]&SPI_CSR_BITS_Msk)!=0) ? XDMAC_CC_DWIDTH_HALFWORD : XDMAC_CC_DWIDTH_BYTE) |
          XDMAC_CC_SIF_AHB_IF1 |
          XDMAC_CC_DIF_AHB_IF0 |
          XDMAC_CC_SAM_FIXED_AM |
          //XDMAC_CC_DAM_INCREMENTED_AM |
          XDMAC_CC_PERID(XDMAIF_Get_ChannelNumber(spi->id, XDMAD_TRANSFER_RX));
      xdmadRxCfg.mbr_bc = 0;
      xdmadRxCfg.mbr_sus = 0;
      xdmadRxCfg.mbr_dus = 0;
      if (XDMAD_ConfigureTransfer(spi->dma, *spi->dma_rx, &xdmadRxCfg, 0, 0, XDMA_INT))
        return ARM_DRIVER_ERROR;
      if (XDMAD_StartTransfer(spi->dma, *spi->dma_rx)) 
        return ARM_DRIVER_ERROR;
    }
    
    if (spi->dma_tx != NULL) {
      // Setup TX 
      xdmadTxCfg.mbr_sa = (uint32_t) spi->xfer->tx_buf;   
      xdmadTxCfg.mbr_da = (uint32_t) &spi->reg->SPI_TDR;
      xdmadTxCfg.mbr_ubc =  XDMA_UBC_NVIEW_NDV0 |
          XDMA_UBC_NDE_FETCH_DIS|
          XDMA_UBC_NSEN_UPDATED | spi->xfer->num;
      xdmadTxCfg.mbr_cfg = XDMAC_CC_TYPE_PER_TRAN |
          XDMAC_CC_MBSIZE_SINGLE |
          XDMAC_CC_DSYNC_MEM2PER |
          XDMAC_CC_CSIZE_CHK_1 |
          (((spi->reg->SPI_CSR[spi->npcs_num]&SPI_CSR_BITS_Msk)!=0) ? XDMAC_CC_DWIDTH_HALFWORD : XDMAC_CC_DWIDTH_BYTE) |
          XDMAC_CC_SIF_AHB_IF0 |
          XDMAC_CC_DIF_AHB_IF1 |
          XDMAC_CC_SAM_INCREMENTED_AM |
          XDMAC_CC_DAM_FIXED_AM |
          XDMAC_CC_PERID(XDMAIF_Get_ChannelNumber(spi->id, XDMAD_TRANSFER_TX));
      xdmadTxCfg.mbr_bc = 0;
      xdmadTxCfg.mbr_sus = 0;
      xdmadTxCfg.mbr_dus = 0;
      if (XDMAD_ConfigureTransfer(spi->dma, *spi->dma_tx, &xdmadTxCfg, 0, 0, XDMA_INT))
        return ARM_DRIVER_ERROR;
      if (XDMAD_StartTransfer(spi->dma, *spi->dma_tx)) 
        return ARM_DRIVER_ERROR;    
    }
    
  } 
  else
#endif  
  { 
    // Configure and enable SPI interrupt
    NVIC_ClearPendingIRQ(spi->irq_num);
    NVIC_SetPriority(spi->irq_num,1);
    NVIC_EnableIRQ(spi->irq_num);     
    spi->reg->SPI_IER = SPI_IER_RDRF | SPI_IER_TXEMPTY;     
  }
  
  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t SPIx_Receive (void *data, uint32_t num, const SPI_RESOURCES *spi)
  \brief       Start receiving data from SPI receiver.
  \param[out]  data  Pointer to buffer for data to receive from SPI receiver
  \param[in]   num   Number of data items to receive
  \param[in]   spi   Pointer to SPI resources
  \return      \ref execution_status
*/
static int32_t SPIx_Receive (void *data, uint32_t num, const SPI_RESOURCES *spi) {
#if (DMA_SPI_EN!=0) 
  sXdmadCfg xdmadTxCfg, xdmadRxCfg;
#endif
  
  if ((data == NULL) || (num == 0U))             { return ARM_DRIVER_ERROR_PARAMETER; }
  if ((spi->info->state & SPI_CONFIGURED) == 0U) { return ARM_DRIVER_ERROR; }
  if ( spi->info->status.busy)                   { return ARM_DRIVER_ERROR_BUSY; }

  // Check if receive pin available
  if ((((spi->io->miso != NULL) && ((spi->info->mode & ARM_SPI_CONTROL_Msk) == ARM_SPI_MODE_MASTER)) ||
       ((spi->io->mosi != NULL) && ((spi->info->mode & ARM_SPI_CONTROL_Msk) == ARM_SPI_MODE_SLAVE ))) == 0U) {
    return ARM_DRIVER_ERROR;
  }

  // Update SPI statuses
  spi->info->status.busy       = 1U;
  spi->info->status.data_lost  = 0U;
  spi->info->status.mode_fault = 0U;

  // Save transfer info
  spi->xfer->rx_buf = (uint8_t *)data;
  spi->xfer->tx_buf = NULL;
  spi->xfer->num    = num;
  spi->xfer->rx_cnt = 0U;
  spi->xfer->tx_cnt = 0U;
    
  // SPI enable
  spi->reg->SPI_CR = SPI_CR_SPIEN;
  
#if (DMA_SPI_EN!=0) 
  if ((spi->dma_tx != NULL)||(spi->dma_rx != NULL)) {   
    
    // Configure and enable DMA interrupt   
    NVIC_ClearPendingIRQ(XDMAC_IRQn);
    NVIC_SetPriority( XDMAC_IRQn ,1);
    NVIC_EnableIRQ(XDMAC_IRQn);
    
    // Clean cache
    SCB_CleanInvalidateDCache();
    
    if (spi->dma_rx != NULL) {
      // Setup RX Link List
      xdmadRxCfg.mbr_sa = (uint32_t) &spi->reg->SPI_RDR;
      xdmadRxCfg.mbr_da = (uint32_t) spi->xfer->rx_buf;
      xdmadRxCfg.mbr_ubc = XDMA_UBC_NVIEW_NDV0 |
          XDMA_UBC_NDE_FETCH_DIS|
          XDMA_UBC_NDEN_UPDATED | spi->xfer->num;
      xdmadRxCfg.mbr_cfg = XDMAC_CC_TYPE_PER_TRAN |
          XDMAC_CC_MBSIZE_SINGLE |
          XDMAC_CC_DSYNC_PER2MEM |
          XDMAC_CC_CSIZE_CHK_1 |
          (((spi->reg->SPI_CSR[spi->npcs_num]&SPI_CSR_BITS_Msk)!=0) ? XDMAC_CC_DWIDTH_HALFWORD : XDMAC_CC_DWIDTH_BYTE) |
          XDMAC_CC_SIF_AHB_IF1 |
          XDMAC_CC_DIF_AHB_IF0 |
          XDMAC_CC_SAM_FIXED_AM |
          XDMAC_CC_DAM_INCREMENTED_AM |
          XDMAC_CC_PERID(XDMAIF_Get_ChannelNumber(spi->id, XDMAD_TRANSFER_RX));
      xdmadRxCfg.mbr_bc = 0;
      xdmadRxCfg.mbr_sus = 0;
      xdmadRxCfg.mbr_dus = 0;
      if (XDMAD_ConfigureTransfer(spi->dma, *spi->dma_rx, &xdmadRxCfg, 0, 0, XDMA_INT))
        return ARM_DRIVER_ERROR;
      if (XDMAD_StartTransfer(spi->dma, *spi->dma_rx)) 
        return ARM_DRIVER_ERROR;
    }
    
    if (spi->dma_tx != NULL) {
      // Setup TX 
      xdmadTxCfg.mbr_sa = (uint32_t) &spi->xfer->def_val;   
      xdmadTxCfg.mbr_da = (uint32_t) &spi->reg->SPI_TDR;
      xdmadTxCfg.mbr_ubc =  XDMA_UBC_NVIEW_NDV0 |
          XDMA_UBC_NDE_FETCH_DIS|
          XDMA_UBC_NSEN_UPDATED | spi->xfer->num;
      xdmadTxCfg.mbr_cfg = XDMAC_CC_TYPE_PER_TRAN |
          XDMAC_CC_MBSIZE_SINGLE |
          XDMAC_CC_DSYNC_MEM2PER |
          XDMAC_CC_CSIZE_CHK_1 |
          (((spi->reg->SPI_CSR[spi->npcs_num]&SPI_CSR_BITS_Msk)!=0) ? XDMAC_CC_DWIDTH_HALFWORD : XDMAC_CC_DWIDTH_BYTE) |
          XDMAC_CC_SIF_AHB_IF0 |
          XDMAC_CC_DIF_AHB_IF1 |
          //XDMAC_CC_SAM_INCREMENTED_AM |
          XDMAC_CC_DAM_FIXED_AM |
          XDMAC_CC_PERID(XDMAIF_Get_ChannelNumber(spi->id, XDMAD_TRANSFER_TX));
      xdmadTxCfg.mbr_bc = 0;
      xdmadTxCfg.mbr_sus = 0;
      xdmadTxCfg.mbr_dus = 0;
      if (XDMAD_ConfigureTransfer(spi->dma, *spi->dma_tx, &xdmadTxCfg, 0, 0, XDMA_INT))
        return ARM_DRIVER_ERROR;
      if (XDMAD_StartTransfer(spi->dma, *spi->dma_tx)) 
        return ARM_DRIVER_ERROR;
    } 
    
  }
  else
#endif  
  {
    // Configure and enable SPI interrupt
    NVIC_ClearPendingIRQ(spi->irq_num);
    NVIC_SetPriority(spi->irq_num,1);
    NVIC_EnableIRQ(spi->irq_num);     
    spi->reg->SPI_IER = SPI_IER_RDRF | SPI_IER_TXEMPTY; 
  }
  
  return ARM_DRIVER_OK;
}

/**
  \fn          uint32_t SPIx_GetDataCount (const SPI_RESOURCES *spi)
  \brief       Get transferred data count.
  \param[in]   spi  Pointer to SPI resources
  \return      number of data items transferred
*/
static uint32_t SPIx_GetDataCount (const SPI_RESOURCES *spi) {
  return (spi->xfer->rx_cnt);
}

/**
  \fn          ARM_SPI_STATUS SPIx_GetStatus (const SPI_RESOURCES *spi)
  \brief       Get SPI status.
  \param[in]   spi  Pointer to SPI resources
  \return      SPI status \ref ARM_SPI_STATUS
*/
static ARM_SPI_STATUS SPIx_GetStatus (const SPI_RESOURCES *spi) {
  return (spi->info->status);
}


/* SPI0 */
#if (RTE_SPI0 != 0)

static int32_t        SPI0_Initialize          (ARM_SPI_SignalEvent_t pSignalEvent)                { return SPIx_Initialize (pSignalEvent, &SPI0_Resources); }
static int32_t        SPI0_Uninitialize        (void)                                              { return SPIx_Uninitialize (&SPI0_Resources); }
static int32_t        SPI0_PowerControl        (ARM_POWER_STATE state)                             { return SPIx_PowerControl (state, &SPI0_Resources); }
static int32_t        SPI0_Send                (const void *data, uint32_t num)                    { return SPIx_Send (data, num, &SPI0_Resources); }
static int32_t        SPI0_Receive             (void *data, uint32_t num)                          { return SPIx_Receive (data, num, &SPI0_Resources); }
static int32_t        SPI0_Transfer            (const void *data_out, void *data_in, uint32_t num) { return SPIx_Transfer (data_out, data_in, num, &SPI0_Resources); }
static uint32_t       SPI0_GetDataCount        (void)                                              { return SPIx_GetDataCount (&SPI0_Resources); }
static int32_t        SPI0_Control             (uint32_t control, uint32_t arg)                    { return SPIx_Control (control, arg, &SPI0_Resources); }
static ARM_SPI_STATUS SPI0_GetStatus           (void)                                              { return SPIx_GetStatus (&SPI0_Resources); }
       void           SPI0_Handler             (void)                                              {        SPIx_IRQHandler (&SPI0_Resources); }
       
ARM_DRIVER_SPI Driver_SPI0 = {
  SPIx_GetVersion,
  SPIx_GetCapabilities,
  SPI0_Initialize,
  SPI0_Uninitialize,
  SPI0_PowerControl,
  SPI0_Send,
  SPI0_Receive,
  SPI0_Transfer,
  SPI0_GetDataCount,
  SPI0_Control,
  SPI0_GetStatus
};
#endif

/* SPI1 */
#if (RTE_SPI1 != 0)

static int32_t        SPI1_Initialize          (ARM_SPI_SignalEvent_t pSignalEvent)                { return SPIx_Initialize (pSignalEvent, &SPI1_Resources); }
static int32_t        SPI1_Uninitialize        (void)                                              { return SPIx_Uninitialize (&SPI1_Resources); }
static int32_t        SPI1_PowerControl        (ARM_POWER_STATE state)                             { return SPIx_PowerControl (state, &SPI1_Resources); }
static int32_t        SPI1_Send                (const void *data, uint32_t num)                    { return SPIx_Send (data, num, &SPI1_Resources); }
static int32_t        SPI1_Receive             (void *data, uint32_t num)                          { return SPIx_Receive (data, num, &SPI1_Resources); }
static int32_t        SPI1_Transfer            (const void *data_out, void *data_in, uint32_t num) { return SPIx_Transfer (data_out, data_in, num, &SPI1_Resources); }
static uint32_t       SPI1_GetDataCount        (void)                                              { return SPIx_GetDataCount (&SPI1_Resources); }
static int32_t        SPI1_Control             (uint32_t control, uint32_t arg)                    { return SPIx_Control (control, arg, &SPI1_Resources); }
static ARM_SPI_STATUS SPI1_GetStatus           (void)                                              { return SPIx_GetStatus (&SPI1_Resources); }
       void           SPI1_Handler             (void)                                              {        SPIx_IRQHandler (&SPI1_Resources); }
       
ARM_DRIVER_SPI Driver_SPI1 = {
  SPIx_GetVersion,
  SPIx_GetCapabilities,
  SPI1_Initialize,
  SPI1_Uninitialize,
  SPI1_PowerControl,
  SPI1_Send,
  SPI1_Receive,
  SPI1_Transfer,
  SPI1_GetDataCount,
  SPI1_Control,
  SPI1_GetStatus
};
#endif

