/* -----------------------------------------------------------------------------
 * Copyright (c) 2015 ARM Ltd.
 *  
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *  
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 * 
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * $Date:        30. April 2015
 * $Revision:    V1.02
 *
 * Project:      CMSIS SPI Driver for Atmel SAM-ESV7
 * -------------------------------------------------------------------------- */
 
#ifndef __SPI_SAM_ESV7_H
#define __SPI_SAM_ESV7_H

#include "RTE_Device.h"
#include "RTE_Components.h"
#include "Driver_SPI.h"
#include "chip.h"
#include <string.h>

#if (defined(RTE_Drivers_SPI) && \
     (RTE_SPI0 == 0) && \
     (RTE_SPI1 == 0))
#error "SPI not configured in RTE_Device.h!"
#endif

// SPI Input/Output Configuration
typedef struct _SPI_IO {
  Pin              *mosi;           // Pointer to MOSI pin configuration
  Pin              *miso;           // Pointer to MISO pin configuration
  Pin              *spck;           // Pointer to SCK pin configuration
  Pin              *npcs;           // Pointer to NPCS pin configuration
} SPI_IO;

// SPI Information (Run-time)
typedef struct _SPI_INFO {
  ARM_SPI_SignalEvent_t cb_event;       // Event Callback
  ARM_SPI_STATUS        status;         // Status flags
  uint8_t               state;          // Current SPI state
  uint32_t              mode;           // Current SPI mode
} SPI_INFO;

// SPI Transfer Information (Run-Time)
typedef struct _SPI_TRANSFER_INFO {
  uint32_t              num;            // Total number of transfers
  uint8_t              *rx_buf;         // Pointer to in data buffer
  uint8_t              *tx_buf;         // Pointer to out data buffer
  uint32_t              rx_cnt;         // Number of data received
  uint32_t              tx_cnt;         // Number of data sent
  uint32_t              dump_val;       // Variable for dumping DMA data
  uint16_t              def_val;        // Default transfer value
} SPI_TRANSFER_INFO;

// SPI Resources definition
typedef struct {
  Spi                  *reg;                  // HW register
  uint32_t              id;                   // Peripheral id
  IRQn_Type             irq_num;              // SPI IRQ Number
  uint8_t               npcs_num;             // NPCS number
  SPI_IO               *io;                   // SPI Input/Output pins
  SPI_INFO             *info;                 // Run-Time Information
  SPI_TRANSFER_INFO    *xfer;                 // SPI transfer information
  sXdmad               *dma;                  // DMA driver instance  
  uint32_t             *dma_tx;               // DMA TX channel 
  uint32_t             *dma_rx;               // DMA RX channel 
} SPI_RESOURCES;

// Current driver status flag definition
#define SPI_INITIALIZED           ((uint8_t)(1U))          // SPI initialized
#define SPI_POWERED               ((uint8_t)(1U << 1))     // SPI powered on
#define SPI_CONFIGURED            ((uint8_t)(1U << 2))     // SPI configured
#define SPI_DATA_LOST             ((uint8_t)(1U << 3))     // SPI data lost occurred
#define SPI_MODE_FAULT            ((uint8_t)(1U << 4))     // SPI mode fault occurred

#define XDMA_INT (XDMAC_CIE_BIE | XDMAC_CIE_DIE | XDMAC_CIE_FIE | XDMAC_CIE_RBIE | XDMAC_CIE_WBIE | XDMAC_CIE_ROIE)

#endif /* __SPI_SAM_ESV7_H */
