/* -----------------------------------------------------------------------------
 * Copyright (c) 2015 ARM Ltd.
 *  
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *  
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 * 
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * $Date:        30. April 2015
 * $Revision:    V1.02
 *
 * Project:      CMSIS USART Driver for Atmel SAM-ESV7
 * -------------------------------------------------------------------------- */

#include "USART_SAM-ESV7.h"

/* ---------------------------------------------------------------------- 
 * Use the following configuration settings in the middleware component
 * to connect to this driver.
 * 
 *   Configuration Setting              Value     UART Interface
 *   ---------------------              -----     --------------
 *   Connect to hardware via RTE_USART# = 0       use USART0
 *   Connect to hardware via RTE_USART# = 1       use USART1
 *   Connect to hardware via RTE_USART# = 2       use USART2
 * -------------------------------------------------------------------- */


#define ARM_USART_DRV_VERSION    ARM_DRIVER_VERSION_MAJOR_MINOR(1,02)  /* driver version */
#define XDMA_INT (XDMAC_CIE_BIE | XDMAC_CIE_DIE | XDMAC_CIE_FIE | XDMAC_CIE_RBIE | XDMAC_CIE_WBIE | XDMAC_CIE_ROIE)

/* Driver Version */
static const ARM_DRIVER_VERSION DriverVersion = {
  ARM_USART_API_VERSION,
  ARM_USART_DRV_VERSION
};

// Driver mode definition
#if   ((RTE_USART0 != 0) && defined(RTE_USART0_IRQ_TX))|| \
      ((RTE_USART1 != 0) && defined(RTE_USART1_IRQ_TX))|| \
      ((RTE_USART2 != 0) && defined(RTE_USART2_IRQ_TX))
#define USART_IRQ_TX

#elif ((RTE_USART0 != 0) && defined(RTE_USART0_POLLING_TX))|| \
      ((RTE_USART1 != 0) && defined(RTE_USART1_POLLING_TX))|| \
      ((RTE_USART2 != 0) && defined(RTE_USART2_POLLING_TX))
#define USART_POLLING_TX
#endif

// Driver mode defintion
#if   ((RTE_USART0 != 0) && defined(RTE_USART0_IRQ_RX))|| \
      ((RTE_USART1 != 0) && defined(RTE_USART1_IRQ_RX))|| \
      ((RTE_USART2 != 0) && defined(RTE_USART2_IRQ_RX))
#define USART_IRQ_RX

#elif ((RTE_USART0 != 0) && defined(RTE_USART0_POLLING_RX))|| \
      ((RTE_USART1 != 0) && defined(RTE_USART1_POLLING_RX))|| \
      ((RTE_USART2 != 0) && defined(RTE_USART2_POLLING_RX))
#define USART_POLLING_RX
#endif

//
//   Functions
//
#if ((RTE_USART0!=0)&((RTE_USART0_DMA_TX!=0)|(RTE_USART0_DMA_RX!=0)))| \
    ((RTE_USART1!=0)&((RTE_USART1_DMA_TX!=0)|(RTE_USART1_DMA_RX!=0)))| \
    ((RTE_USART2!=0)&((RTE_USART2_DMA_TX!=0)|(RTE_USART2_DMA_RX!=0)))
#define DMA_USART_EN 1
#else
#define DMA_USART_EN 0
#endif
static int32_t USARTx_Receive (      void            *data,
                                     uint32_t         num,
                               const USART_RESOURCES *usart);

#if (DMA_USART_EN!=0)
static sXdmad USARTx_Dma;
#endif

/* USART0 */
#if (RTE_USART0 != 0)
// USART1 Run-Time Information
static USART_INFO          USART0_Info = {0};
static USART_TRANSFER_INFO USART0_TransferInfo = {0};

// USART0 Configure pins
#if RTE_USART0_TX_PORT_ID == 1
static Pin USART0_TX_PORT = RTE_USART0_TX_PORT;
#endif
#if RTE_USART0_RX_PORT_ID == 1
static Pin USART0_RX_PORT = RTE_USART0_RX_PORT;
#endif
#if RTE_USART0_SCK_PORT_ID == 1
static Pin USART0_SCK_PORT  = RTE_USART0_SCK_PORT;
#endif
#if RTE_USART0_RTS_PORT_ID == 1
static Pin USART0_RTS_PORT  = RTE_USART0_RTS_PORT;
#endif
#if RTE_USART0_CTS_PORT_ID == 1
static Pin USART0_CTS_PORT  = RTE_USART0_CTS_PORT;
#endif
#if RTE_USART0_DCD_PORT_ID == 1
static Pin USART0_DCD_PORT  = RTE_USART0_DCD_PORT;
#endif
#if RTE_USART0_DSR_PORT_ID == 1
static Pin USART0_DSR_PORT  = RTE_USART0_DSR_PORT;
#endif
#if RTE_USART0_DTR_PORT_ID == 1
static Pin USART0_DTR_PORT  = RTE_USART0_DTR_PORT;
#endif
#if RTE_USART0_RI_PORT_ID == 1
static Pin USART0_RI_PORT = RTE_USART0_RI_PORT;
#endif


static USART_IO USART0_Pins = {
#if RTE_USART0_TX_PORT_ID == 1
  &USART0_TX_PORT,
#else
  NULL,
#endif
#if RTE_USART0_RX_PORT_ID == 1
  &USART0_RX_PORT,
#else
  NULL,
#endif
#if RTE_USART0_SCK_PORT_ID == 1
  &USART0_SCK_PORT,
#else
  NULL,
#endif
#if RTE_USART0_RTS_PORT_ID == 1
  &USART0_RTS_PORT,
#else
  NULL,
#endif
#if RTE_USART0_CTS_PORT_ID == 1
  &USART0_CTS_PORT,
#else
  NULL,
#endif
#if RTE_USART0_DCD_PORT_ID == 1
  &USART0_DCD_PORT,
#else
  NULL,
#endif
#if RTE_USART0_DSR_PORT_ID == 1
  &USART0_DSR_PORT,
#else
  NULL,
#endif
#if RTE_USART0_DTR_PORT_ID == 1
  &USART0_DTR_PORT,
#else
  NULL,
#endif
#if RTE_USART0_RI_PORT_ID == 1
  &USART0_RI_PORT,
#else
  NULL,
#endif
  NULL,
};

#if (RTE_USART0_DMA_TX!=0)
uint32_t USART0_DmaTxChannel;
#endif
#if (RTE_USART0_DMA_RX!=0)
uint32_t USART0_DmaRxChannel;
#endif

// USART0 Resources
static const USART_RESOURCES USART0_Resources = {
  {
    1, /* supports UART (Asynchronous) mode */
    1, /* supports Synchronous Master mode */
    0, /* supports Synchronous Slave mode */
    0, /* supports UART Single-wire mode */
    0, /* supports UART IrDA mode */
    1, /* supports UART Smart Card mode */
    1, /* Smart Card Clock generator available */
#if RTE_USART0_RTS_PORT_ID == 1
    1, /* RTS Flow Control available */
#else
    0, /* RTS Flow Control available */
#endif
#if RTE_USART0_CTS_PORT_ID == 1
    1, /* CTS Flow Control available */
#else
    0, /* CTS Flow Control available */
#endif
    0, /* Transmit completed event: \ref ARM_USART_EVENT_TX_COMPLETE */
    0, /* Signal receive character timeout event: \ref ARM_USART_EVENT_RX_TIMEOUT */
#if RTE_USART0_RTS_PORT_ID == 1
    1, /* RTS Line: 0=not available, 1=available */
#else
    0, /* RTS Line: 0=not available, 1=available */
#endif
#if RTE_USART0_CTS_PORT_ID == 1
    1, /* CTS Line: 0=not available, 1=available */
#else
    0, /* CTS Line: 0=not available, 1=available */
#endif
    0, /* DTR Line: 0=not available, 1=available */
    0, /* DSR Line: 0=not available, 1=available */
    0, /* DCD Line: 0=not available, 1=available */
    0, /* RI Line: 0=not available, 1=available */
    0, /* Signal CTS change event: \ref ARM_USART_EVENT_CTS */
    0, /* Signal DSR change event: \ref ARM_USART_EVENT_DSR */
    0, /* Signal DCD change event: \ref ARM_USART_EVENT_DCD */
    0  /* Signal RI change event: \ref ARM_USART_EVENT_RI */
  },
  USART0,
  ID_USART0,
  USART0_IRQn,
  &USART0_Pins,
  &USART0_Info,
  &USART0_TransferInfo,
#if (RTE_USART0_DMA_TX!=0)|(RTE_USART0_DMA_RX!=0)
  &USARTx_Dma,
#else
  NULL,
#endif
#if (RTE_USART0_DMA_TX!=0)
  &USART0_DmaTxChannel,
#else
  NULL,
#endif
#if (RTE_USART0_DMA_RX!=0)
  &USART0_DmaRxChannel,
#else
  NULL,
#endif
};

#endif /* #if (RTE_USART0 != 0) ... */

/* USART1 */
#if (RTE_USART1 != 0)
// USART1 Run-Time Information
static USART_INFO          USART1_Info = {0};
static USART_TRANSFER_INFO USART1_TransferInfo = {0};

// USART1 Configure pins
#if RTE_USART1_TX_PORT_ID == 1
static Pin USART1_TX_PORT = RTE_USART1_TX_PORT;
#endif
#if RTE_USART1_RX_PORT_ID == 1
static Pin USART1_RX_PORT = RTE_USART1_RX_PORT;
#endif
#if RTE_USART1_SCK_PORT_ID == 1
static Pin USART1_SCK_PORT  = RTE_USART1_SCK_PORT;
#endif
#if RTE_USART1_RTS_PORT_ID == 1
static Pin USART1_RTS_PORT  = RTE_USART1_RTS_PORT;
#endif
#if RTE_USART1_CTS_PORT_ID == 1
static Pin USART1_CTS_PORT  = RTE_USART1_CTS_PORT;
#endif
#if RTE_USART1_DCD_PORT_ID == 1
static Pin USART1_DCD_PORT  = RTE_USART1_DCD_PORT;
#endif
#if RTE_USART1_DSR_PORT_ID == 1
static Pin USART1_DSR_PORT  = RTE_USART1_DSR_PORT;
#endif
#if RTE_USART1_DTR_PORT_ID == 1
static Pin USART1_DTR_PORT  = RTE_USART1_DTR_PORT;
#endif
#if RTE_USART1_RI_PORT_ID == 1
static Pin USART1_RI_PORT = RTE_USART1_RI_PORT;
#endif
#if RTE_USART1_LONCOL_PORT_ID == 1
static Pin USART1_LONCOL_PORT = RTE_USART1_LONCOL_PORT;
#endif


static USART_IO USART1_Pins = {
#if RTE_USART1_TX_PORT_ID == 1
  &USART1_TX_PORT,
#else
  NULL,
#endif
#if RTE_USART1_RX_PORT_ID == 1
  &USART1_RX_PORT,
#else
  NULL,
#endif
#if RTE_USART1_SCK_PORT_ID == 1
  &USART1_SCK_PORT,
#else
  NULL,
#endif
#if RTE_USART1_RTS_PORT_ID == 1
  &USART1_RTS_PORT,
#else
  NULL,
#endif
#if RTE_USART1_CTS_PORT_ID == 1
  &USART1_CTS_PORT,
#else
  NULL,
#endif
#if RTE_USART1_DCD_PORT_ID == 1
  &USART1_DCD_PORT,
#else
  NULL,
#endif
#if RTE_USART1_DSR_PORT_ID == 1
  &USART1_DSR_PORT,
#else
  NULL,
#endif
#if RTE_USART1_DTR_PORT_ID == 1
  &USART1_DTR_PORT,
#else
  NULL,
#endif
#if RTE_USART1_RI_PORT_ID == 1
  &USART1_RI_PORT,
#else
  NULL,
#endif
#if RTE_USART1_LONCOL_PORT_ID == 1
  &USART1_LONCOL_PORT,
#else
  NULL,
#endif
};

#if (RTE_USART1_DMA_TX!=0)
uint32_t USART1_DmaTxChannel;
#endif
#if (RTE_USART1_DMA_RX!=0)
uint32_t USART1_DmaRxChannel;
#endif

// USART1 Resources
static const USART_RESOURCES USART1_Resources = {
  {
    1, /* supports UART (Asynchronous) mode */
    1, /* supports Synchronous Master mode */
    0, /* supports Synchronous Slave mode */
    0, /* supports UART Single-wire mode */
    0, /* supports UART IrDA mode */
    1, /* supports UART Smart Card mode */
#if RTE_USART1_RTS_PORT_ID == 1
    1, /* RTS Flow Control available */
#else
    0, /* RTS Flow Control available */
#endif
#if RTE_USART1_CTS_PORT_ID == 1
    1, /* CTS Flow Control available */
#else
    0, /* CTS Flow Control available */
#endif
    0, /* Transmit completed event: \ref ARM_USART_EVENT_TX_COMPLETE */
    0, /* Signal receive character timeout event: \ref ARM_USART_EVENT_RX_TIMEOUT */
#if RTE_USART1_RTS_PORT_ID == 1
    1, /* RTS Line: 0=not available, 1=available */
#else
    0, /* RTS Line: 0=not available, 1=available */
#endif
#if RTE_USART1_CTS_PORT_ID == 1
    1, /* CTS Line: 0=not available, 1=available */
#else
    0, /* CTS Line: 0=not available, 1=available */
#endif
    0, /* DTR Line: 0=not available, 1=available */
    0, /* DSR Line: 0=not available, 1=available */
    0, /* DCD Line: 0=not available, 1=available */
    0, /* RI Line: 0=not available, 1=available */
    0, /* Signal CTS change event: \ref ARM_USART_EVENT_CTS */
    0, /* Signal DSR change event: \ref ARM_USART_EVENT_DSR */
    0, /* Signal DCD change event: \ref ARM_USART_EVENT_DCD */
    0  /* Signal RI change event: \ref ARM_USART_EVENT_RI */
  },
  USART1,
  ID_USART1,
  USART1_IRQn,
  &USART1_Pins,
  &USART1_Info,
  &USART1_TransferInfo,
#if (RTE_USART1_DMA_TX!=0)|(RTE_USART1_DMA_RX!=0)
  &USARTx_Dma,
#else
  NULL,
#endif
#if (RTE_USART1_DMA_TX!=0)
  &USART1_DmaTxChannel,
#else
  NULL,
#endif
#if (RTE_USART1_DMA_RX!=0)
  &USART1_DmaRxChannel,
#else
  NULL,
#endif
};

#endif /* #if (RTE_USART1 != 0) ... */

/* USART2 */
#if (RTE_USART2 != 0)
// USART2 Run-Time Information
static USART_INFO          USART2_Info = {0};
static USART_TRANSFER_INFO USART2_TransferInfo = {0};

// USART2 Configure pins
#if RTE_USART2_TX_PORT_ID == 1
static Pin USART2_TX_PORT = RTE_USART2_TX_PORT;
#endif
#if RTE_USART2_RX_PORT_ID == 1
static Pin USART2_RX_PORT = RTE_USART2_RX_PORT;
#endif
#if RTE_USART2_SCK_PORT_ID == 1
static Pin USART2_SCK_PORT  = RTE_USART2_SCK_PORT;
#endif
#if RTE_USART2_RTS_PORT_ID == 1
static Pin USART2_RTS_PORT  = RTE_USART2_RTS_PORT;
#endif
#if RTE_USART2_CTS_PORT_ID == 1
static Pin USART2_CTS_PORT  = RTE_USART2_CTS_PORT;
#endif
#if RTE_USART2_DCD_PORT_ID == 1
static Pin USART2_DCD_PORT  = RTE_USART2_DCD_PORT;
#endif
#if RTE_USART2_DSR_PORT_ID == 1
static Pin USART2_DSR_PORT  = RTE_USART2_DSR_PORT;
#endif
#if RTE_USART2_DTR_PORT_ID == 1
static Pin USART2_DTR_PORT  = RTE_USART2_DTR_PORT;
#endif
#if RTE_USART2_RI_PORT_ID == 1
static Pin USART2_RI_PORT = RTE_USART2_RI_PORT;
#endif

static USART_IO USART2_Pins = {
#if RTE_USART2_TX_PORT_ID == 1
  &USART2_TX_PORT,
#else
  NULL,
#endif
#if RTE_USART2_RX_PORT_ID == 1
  &USART2_RX_PORT,
#else
  NULL,
#endif
#if RTE_USART2_SCK_PORT_ID == 1
  &USART2_SCK_PORT,
#else
  NULL,
#endif
#if RTE_USART2_RTS_PORT_ID == 1
  &USART2_RTS_PORT,
#else
  NULL,
#endif
#if RTE_USART2_CTS_PORT_ID == 1
  &USART2_CTS_PORT,
#else
  NULL,
#endif
#if RTE_USART2_DCD_PORT_ID == 1
  &USART2_DCD_PORT,
#else
  NULL,
#endif
#if RTE_USART2_DSR_PORT_ID == 1
  &USART2_DSR_PORT,
#else
  NULL,
#endif
#if RTE_USART2_DTR_PORT_ID == 1
  &USART2_DTR_PORT,
#else
  NULL,
#endif
#if RTE_USART2_RI_PORT_ID == 1
  &USART2_RI_PORT,
#else
  NULL,
#endif
  NULL,
};

#if (RTE_USART2_DMA_TX!=0)
uint32_t USART2_DmaTxChannel;
#endif
#if (RTE_USART2_DMA_RX!=0)
uint32_t USART2_DmaRxChannel;
#endif

// USART2 Resources
static const USART_RESOURCES USART2_Resources = {
  {
    1, /* supports UART (Asynchronous) mode */
    1, /* supports Synchronous Master mode */
    0, /* supports Synchronous Slave mode */
    0, /* supports UART Single-wire mode */
    0, /* supports UART IrDA mode */
    1, /* supports UART Smart Card mode */
    1, /* Smart Card Clock generator available */
#if RTE_USART2_RTS_PORT_ID == 1
    1, /* RTS Flow Control available */
#else
    0, /* RTS Flow Control available */
#endif
#if RTE_USART2_CTS_PORT_ID == 1
    1, /* CTS Flow Control available */
#else
    0, /* CTS Flow Control available */
#endif
    0, /* Transmit completed event: \ref ARM_USART_EVENT_TX_COMPLETE */
    0, /* Signal receive character timeout event: \ref ARM_USART_EVENT_RX_TIMEOUT */
#if RTE_USART2_RTS_PORT_ID == 1
    1, /* RTS Line: 0=not available, 1=available */
#else
    0, /* RTS Line: 0=not available, 1=available */
#endif
#if RTE_USART2_CTS_PORT_ID == 1
    1, /* CTS Line: 0=not available, 1=available */
#else
    0, /* CTS Line: 0=not available, 1=available */
#endif
    0, /* DTR Line: 0=not available, 1=available */
    0, /* DSR Line: 0=not available, 1=available */
    0, /* DCD Line: 0=not available, 1=available */
    0, /* RI Line: 0=not available, 1=available */
    0, /* Signal CTS change event: \ref ARM_USART_EVENT_CTS */
    0, /* Signal DSR change event: \ref ARM_USART_EVENT_DSR */
    0, /* Signal DCD change event: \ref ARM_USART_EVENT_DCD */
    0  /* Signal RI change event: \ref ARM_USART_EVENT_RI */
  },
  USART2,
  ID_USART2,
  USART2_IRQn,
  &USART2_Pins,
  &USART2_Info,
  &USART2_TransferInfo,
#if (RTE_USART2_DMA_TX!=0)|(RTE_USART2_DMA_RX!=0)
  &USARTx_Dma,
#else
  NULL,
#endif
#if (RTE_USART2_DMA_TX!=0)
  &USART2_DmaTxChannel,
#else
  NULL,
#endif
#if (RTE_USART2_DMA_RX!=0)
  &USART2_DmaRxChannel,
#else
  NULL,
#endif
};

#endif /* #if (RTE_USART2 != 0) ... */

#if (DMA_USART_EN!=0)
/* void XDMAC_Handler(void) */
/* { */
/*     XDMAD_Handler(USART1_Resources.dma); */
/* } */

/* USARTx DMA Callback */
static void USARTx_Rx_Callback(uint32_t channel, const USART_RESOURCES *usart)
{
  uint32_t event = 0;

  if (channel != *usart->dma_rx) return;

  // Update USART info
  usart->xfer->rx_num = 0U;
  usart->info->status.rx_busy = 0U;
  event |= ARM_USART_EVENT_RECEIVE_COMPLETE;
  usart->info->cb_event(event);

  memory_barrier();
}

/* USARTx DMA Callback */
static void USARTx_Tx_Callback(uint32_t channel, const USART_RESOURCES *usart)
{
  uint32_t event = 0;

  if (channel != *usart->dma_tx) return;

  // Update USART info
  usart->xfer->tx_num = 0U;
  usart->info->status.tx_busy = 0U;
  event |= ARM_USART_EVENT_SEND_COMPLETE;
  usart->info->state &= ~USART_SEND_ACTIVE;
  usart->info->cb_event(event);

  memory_barrier();
}
#endif

/* USART IRQ Handler */
void USARTx_IRQHandler (const USART_RESOURCES *usart) {
#if defined(USART_IRQ_RX) || defined(USART_IRQ_TX)
  uint32_t event = 0;
#endif
  uint32_t status = usart->reg->US_CSR;

#ifdef USART_IRQ_RX
  // Error
  if ((status & (US_CSR_RXBRK | US_CSR_OVRE | US_CSR_FRAME | US_CSR_PARE | US_CSR_TIMEOUT)) != 0 ) {
    usart->reg->US_IDR = US_IDR_RXRDY;
    usart->xfer->rx_num = 0U;
    usart->info->status.rx_busy = 0U;
    if (status & (US_CSR_RXBRK))
  event |= ARM_USART_EVENT_RX_BREAK;
    if (status & (US_CSR_OVRE))
  event |= ARM_USART_EVENT_RX_OVERFLOW;
    if (status & (US_CSR_FRAME))
  event |= ARM_USART_EVENT_RX_FRAMING_ERROR;
    if (status & (US_CSR_PARE))
  event |= ARM_USART_EVENT_RX_PARITY_ERROR;
    if (status & (US_CSR_TIMEOUT))
  event |= ARM_USART_EVENT_RX_TIMEOUT;
    usart->info->cb_event(event);
  }

  // RX
  if ((status & US_CSR_RXRDY) != 0 ) {
    if (usart->xfer->rx_buf != NULL)
      usart->xfer->rx_buf[usart->xfer->rx_cnt++] = usart->reg->US_RHR & 0xFFFF;

    // Transfer is completed
    if (usart->xfer->rx_cnt >= usart->xfer->rx_num) {
      usart->reg->US_IDR = US_IDR_RXRDY;
      usart->xfer->rx_num = 0U;
      usart->info->status.rx_busy = 0U;
      event |= ARM_USART_EVENT_RECEIVE_COMPLETE;
      usart->info->cb_event(event);
    }
  }
#endif
  
#ifdef USART_IRQ_TX
  // TX
  if ((status & (US_CSR_TXRDY)) != 0 ) {
    if (usart->xfer->tx_num > 0) {
      // Write THR register
      if (usart->xfer->tx_buf != NULL)
          usart->reg->US_THR = usart->xfer->tx_buf[usart->xfer->tx_cnt];
      else
          usart->reg->US_THR = usart->xfer->def_val;
      usart->xfer->tx_cnt++;

      // Transfer is completed
      if (usart->xfer->tx_cnt >= usart->xfer->tx_num) {
          usart->reg->US_IDR = US_IDR_TXRDY;
          usart->xfer->tx_num = 0U;
          usart->info->status.tx_busy = 0U;
          event |= ARM_USART_EVENT_SEND_COMPLETE;
          usart->info->state &= ~USART_SEND_ACTIVE;
          usart->info->cb_event(event);
      }
    }
  }
#endif
  
  // If no more communication to expect: disable the interruption
  if (usart->info->status.tx_busy  == 0 && usart->info->status.rx_busy == 0) {
    NVIC_ClearPendingIRQ(usart->irq_num);
    NVIC_DisableIRQ(usart->irq_num);
  }
}


/**
  \fn          ARM_DRIVER_VERSION USARTx_GetVersion (void)
  \brief       Get driver version.
  \return      \ref ARM_DRIVER_VERSION
*/
ARM_DRIVER_VERSION USARTx_GetVersion(void)
{
  return DriverVersion;
}

/**
  \fn          ARM_USART_CAPABILITIES USARTx_GetCapabilities (const USART_RESOURCES *usart)
  \brief       Get driver capabilities
  \param[in]   usart     Pointer to USART resources
  \return      \ref ARM_USART_CAPABILITIES
*/
ARM_USART_CAPABILITIES USARTx_GetCapabilities(const USART_RESOURCES *usart)
{
  return usart->capabilities;
}

/**
  \fn          int32_t USARTx_Initialize (      ARM_USART_SignalEvent_t  cb_event
                                          const USART_RESOURCES         *usart)
  \brief       Initialize USART Interface.
  \param[in]   cb_event  Pointer to \ref ARM_USART_SignalEvent
  \param[in]   usart     Pointer to USART resources
  \return      \ref execution_status
*/
int32_t USARTx_Initialize(ARM_USART_SignalEvent_t cb_event, const USART_RESOURCES *usart)
{
  if (usart->info->state & USART_POWERED) {
    // Device is powered - could not be re-initialized
    return ARM_DRIVER_ERROR;
  }

  if (usart->info->state & USART_INITIALIZED) {
    // Driver is already initialized
    return ARM_DRIVER_OK;
  }

  // Initialize USART Run-time Resources
  usart->info->cb_event = cb_event;

  usart->info->status.tx_busy          = 0U;
  usart->info->status.rx_busy          = 0U;
  usart->info->status.tx_underflow     = 0U;
  usart->info->status.rx_overflow      = 0U;
  usart->info->status.rx_break         = 0U;
  usart->info->status.rx_framing_error = 0U;
  usart->info->status.rx_parity_error  = 0U;

  usart->info->mode = 0U;

  // Disable the MATRIX registers write protection
  MATRIX->MATRIX_WPMR  = MATRIX_WPMR_WPKEY_PASSWD;
  MATRIX->CCFG_SYSIO |= CCFG_SYSIO_SYSIO4;
  
  // Clear transfer information
  memset(usart->xfer, 0, sizeof(USART_TRANSFER_INFO));
 
  // Pin configuration
  if (usart->io->tx)
    PIO_Configure(usart->io->tx, 1);
  if (usart->io->rx)
    PIO_Configure(usart->io->rx, 1);
  if (usart->io->sck)
    PIO_Configure(usart->io->sck, 1);
  if (usart->io->rts)
    PIO_Configure(usart->io->rts, 1);
  if (usart->io->cts)
    PIO_Configure(usart->io->cts, 1);
  if (usart->io->dcd)
    PIO_Configure(usart->io->dcd, 1);
  if (usart->io->dsr)
    PIO_Configure(usart->io->dsr, 1);
  if (usart->io->dtr)
    PIO_Configure(usart->io->dtr, 1);
  if (usart->io->ri)
    PIO_Configure(usart->io->ri, 1);
  if (usart->io->lcol)
    PIO_Configure(usart->io->lcol, 1);

  usart->info->state = USART_INITIALIZED;

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USARTx_Uninitialize (const USART_RESOURCES *usart)
  \brief       De-initialize USART Interface.
  \param[in]   usart     Pointer to USART resources
  \return      \ref execution_status
*/
int32_t USARTx_Uninitialize(const USART_RESOURCES *usart)
{
  usart->info->state = 0; // USART is uninitiliazed

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USARTx_PowerControl (      ARM_POWER_STATE  state,
                                            const USART_RESOURCES *usart)
  \brief       Control USART Interface Power.
  \param[in]   state  Power state
  \param[in]   usart  Pointer to USART resources
  \return      \ref execution_status
*/
int32_t USARTx_PowerControl(ARM_POWER_STATE state, const USART_RESOURCES *usart)
{
  switch (state) {
    case ARM_POWER_OFF:

#if (DMA_USART_EN!=0)
      if ((usart->dma_tx != NULL)||(usart->dma_rx != NULL)) {
        // Disable DMA Interrupts
        NVIC_ClearPendingIRQ(XDMAC_IRQn);
        NVIC_DisableIRQ(XDMAC_IRQn);

        if (usart->dma_tx != NULL) {
          XDMAD_FreeChannel(usart->dma, *usart->dma_tx);
        }
        if (usart->dma_rx != NULL) {
          XDMAD_FreeChannel(usart->dma, *usart->dma_rx);
        }
      }
      else
#endif
      {
        // Disable USART Interrupts
        NVIC_ClearPendingIRQ(usart->irq_num);
        NVIC_DisableIRQ(usart->irq_num);
        usart->reg->US_IDR = US_IDR_RXRDY | US_IDR_TXRDY;
      }

      // Disable USART peripheral
      usart->reg->US_CR = US_CR_TXDIS;

      // Disable USART clock
      PMC_DisablePeripheral(usart->id);

      usart->info->state &= ~USART_POWERED;
      break;

    case ARM_POWER_FULL:
      if ((usart->info->state & USART_INITIALIZED) == 0U) { return ARM_DRIVER_ERROR; }
      if (usart->info->status.rx_busy == 1U) { return ARM_DRIVER_ERROR_BUSY; }
      if (usart->info->state & USART_SEND_ACTIVE) { return ARM_DRIVER_ERROR_BUSY; }

      if (usart->info->state & USART_POWERED) {
        if (((usart->reg->US_CR & US_CR_TXEN) != 0) && ((usart->reg->US_CSR & US_CSR_TXRDY) == 0U)) {
          // USART is powered, transmiter is enabled and Transmission is not complete
          return ARM_DRIVER_ERROR_BUSY;
        }
      }

      // Check if USART already powered
      if (usart->info->state & USART_POWERED) {
        return ARM_DRIVER_OK;
      }
      
      // Enable peripheral's clock
      PMC_EnablePeripheral(usart->id);

#if (DMA_USART_EN!=0)
      if ((usart->dma_tx != NULL)||(usart->dma_rx != NULL)) {
        // DMA Driver initialize
        XDMAD_Initialize(usart->dma, 0);
        
        /* Configure and enable interrupt on RC compare */
        NVIC_ClearPendingIRQ(XDMAC_IRQn);
        NVIC_SetPriority( XDMAC_IRQn ,1);
        NVIC_EnableIRQ(XDMAC_IRQn);
      }

      if (usart->dma_tx != NULL) {
        XDMAD_FreeChannel(usart->dma, *usart->dma_tx);
      } 
      // Allocate a DMA channel for USART0/1 TX
      *usart->dma_tx = XDMAD_AllocateChannel(usart->dma, XDMAD_TRANSFER_MEMORY, usart->id);
      if (*usart->dma_tx == XDMAD_ALLOC_FAILED) return ARM_DRIVER_ERROR;
      // Setup callbacks for USART0/1 TX
      XDMAD_SetCallback(usart->dma, *usart->dma_tx, (XdmadTransferCallback)USARTx_Tx_Callback,(void*)usart);
      if (XDMAD_PrepareChannel(usart->dma, *usart->dma_tx)) return ARM_DRIVER_ERROR;

      if (usart->dma_rx != NULL) {
        XDMAD_FreeChannel(usart->dma, *usart->dma_rx);
      }
      // Allocate a DMA channel for USART0/1 RX
      *usart->dma_rx = XDMAD_AllocateChannel(usart->dma, usart->id, XDMAD_TRANSFER_MEMORY);
      if (*usart->dma_rx == XDMAD_ALLOC_FAILED) return ARM_DRIVER_ERROR;
      // Setup callbacks for USART0/1 RX
      XDMAD_SetCallback(usart->dma, *usart->dma_rx, (XdmadTransferCallback)USARTx_Rx_Callback,(void*)usart);
      if (XDMAD_PrepareChannel(usart->dma, *usart->dma_rx)) return ARM_DRIVER_ERROR;
#endif

      usart->info->state = USART_POWERED | USART_INITIALIZED;
      break;

    case ARM_POWER_LOW:
    default:
      return ARM_DRIVER_ERROR_UNSUPPORTED;
  }

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USARTx_Send (const void            *data,
                                          uint32_t         num,
                                    const USART_RESOURCES *usart)
  \brief       Start sending data to USART transmitter.
  \param[in]   data  Pointer to buffer with data to send to USART transmitter
  \param[in]   num   Number of data items to send
  \param[in]   usart Pointer to USART resources
  \return      \ref execution_status
*/
int32_t USARTx_Send(const void *data, uint32_t num, const USART_RESOURCES *usart)
{
  int32_t stat;
#if (DMA_USART_EN!=0)
  uint32_t source_inc = XDMAC_CC_SAM_INCREMENTED_AM;
#endif  
  if ((data == NULL) || (num == 0U)) {
    // Invalid parameters
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if ((usart->info->state & USART_CONFIGURED) == 0U) {
    // USART is not configured (mode not selected)
    return ARM_DRIVER_ERROR;
  }

  if (usart->info->state & USART_SEND_ACTIVE) {
    // Send is not completed yet
    return ARM_DRIVER_ERROR_BUSY;
  }

  // Set Send active flag
  usart->info->state |= USART_SEND_ACTIVE;

  // Save transmit buffer info
  usart->xfer->tx_buf = (uint8_t *)data;
  usart->xfer->tx_num = num;
  usart->xfer->tx_cnt = 0U;
  usart->info->status.tx_busy = 1U;

    // Synchronous mode
  if (usart->info->mode == ARM_USART_MODE_SYNCHRONOUS_MASTER) {
    if (usart->xfer->sync_mode == 0U) {
      usart->xfer->sync_mode = USART_SYNC_MODE_TX;
      // Start dummy reads
      stat = USARTx_Receive (&usart->xfer->dump_val, num, usart);
      if (stat == ARM_DRIVER_ERROR_BUSY) { return ARM_DRIVER_ERROR_BUSY; }

#if (DMA_USART_EN!=0)
    } else {
      if (usart->xfer->sync_mode == USART_SYNC_MODE_RX) {
        // Dummy DMA writes (do not increment source address)
        source_inc = XDMAC_CC_SAM_FIXED_AM;
      }
#endif
    }
  }

#if (DMA_USART_EN!=0)
  if (usart->dma_tx != NULL) {
    sXdmadCfg xdmadTxCfg;

    // Configure and enable DMA interrupt   
    NVIC_ClearPendingIRQ(XDMAC_IRQn);
    NVIC_SetPriority( XDMAC_IRQn ,1);
    NVIC_EnableIRQ(XDMAC_IRQn);
    
    // Setup TX
    xdmadTxCfg.mbr_sa = (uint32_t) usart->xfer->tx_buf;
    xdmadTxCfg.mbr_da = (uint32_t) &usart->reg->US_THR;
    xdmadTxCfg.mbr_ubc = XDMA_UBC_NVIEW_NDV0 |
                         XDMA_UBC_NDE_FETCH_DIS|
                         XDMA_UBC_NSEN_UPDATED | usart->xfer->tx_num;
                         xdmadTxCfg.mbr_cfg = XDMAC_CC_TYPE_PER_TRAN |
                         XDMAC_CC_MBSIZE_SINGLE |
                         XDMAC_CC_DSYNC_MEM2PER |
                         XDMAC_CC_CSIZE_CHK_1 |
                         XDMAC_CC_DWIDTH_BYTE|
                         XDMAC_CC_SIF_AHB_IF0 |
                         XDMAC_CC_DIF_AHB_IF1 |
                         source_inc |
                         XDMAC_CC_DAM_FIXED_AM |
                         XDMAC_CC_PERID(XDMAIF_Get_ChannelNumber(usart->id, XDMAD_TRANSFER_TX));
    xdmadTxCfg.mbr_bc = 0;
    xdmadTxCfg.mbr_sus = 0;
    xdmadTxCfg.mbr_dus = 0;
    xdmadTxCfg.mbr_ds= 0;

    SCB_CleanInvalidateDCache();
    if (XDMAD_ConfigureTransfer(usart->dma, *usart->dma_tx, &xdmadTxCfg, 0, 0, XDMA_INT))
      return ARM_DRIVER_ERROR;

    if (XDMAD_StartTransfer(usart->dma, *usart->dma_tx))
      return ARM_DRIVER_ERROR;
  }
  else
#endif
#if defined(USART_IRQ_TX)
  {
        // Configure and enable SPI interrupt
    NVIC_ClearPendingIRQ(usart->irq_num);
    NVIC_SetPriority(usart->irq_num,1);
    NVIC_EnableIRQ(usart->irq_num);
    usart->reg->US_IER = US_IER_TXRDY;;
  }
#else
  {
    while ((usart->reg->US_CSR & US_CSR_TXRDY) == 0);
      
    while (usart->xfer->tx_num--) {
      usart->reg->US_THR = usart->xfer->tx_buf[usart->xfer->tx_cnt++];
      while ((usart->reg->US_CSR & US_CSR_TXEMPTY) == 0);
    }

    usart->info->cb_event(ARM_USART_EVENT_SEND_COMPLETE);
            
    // Set RX busy flag
    usart->info->status.tx_busy = 0U;

    usart->info->state &= ~USART_SEND_ACTIVE;
  }
#endif
  
  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USARTx_Receive (      void            *data,
                                             uint32_t         num,
                                       const USART_RESOURCES *usart)
  \brief       Start receiving data from USART receiver.
  \param[out]  data  Pointer to buffer for data to receive from USART receiver
  \param[in]   num   Number of data items to receive
  \param[in]   usart Pointer to USART resources
  \return      \ref execution_status
*/
int32_t USARTx_Receive(void *data, uint32_t num, const USART_RESOURCES *usart)
{
  int32_t stat;

  if ((data == NULL) || (num == 0U)) {
    // Invalid parameters
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if ((usart->info->state & USART_CONFIGURED) == 0U) {
    // USART is not configured (mode not selected)
    return ARM_DRIVER_ERROR;
  }

  // Check if receiver is busy
  if (usart->info->status.rx_busy == 1U) {
    return ARM_DRIVER_ERROR_BUSY;
  }

  // Save number of data to be received
  usart->xfer->rx_num = num;

  // Clear RX statuses
  usart->info->status.rx_break          = 0U;
  usart->info->status.rx_framing_error  = 0U;
  usart->info->status.rx_overflow       = 0U;
  usart->info->status.rx_parity_error   = 0U;

  // Save receive buffer info
  // Save transfer info
  usart->xfer->rx_buf = (uint8_t *)data;
  usart->xfer->rx_num = num;
  usart->xfer->rx_cnt = 0U;

  // Set RX busy flag
  usart->info->status.rx_busy = 1U;

#if (DMA_USART_EN!=0)
  if (usart->dma_rx != NULL) {
    sXdmadCfg xdmadRxCfg;

    // Configure and enable DMA interrupt   
    NVIC_ClearPendingIRQ(XDMAC_IRQn);
    NVIC_SetPriority( XDMAC_IRQn ,1);
    NVIC_EnableIRQ(XDMAC_IRQn);
    
    // Setup RX Link List
    xdmadRxCfg.mbr_sa = (uint32_t) &usart->reg->US_RHR;
    xdmadRxCfg.mbr_da = (uint32_t) usart->xfer->rx_buf;
    xdmadRxCfg.mbr_ubc = XDMA_UBC_NVIEW_NDV0 |
                         XDMA_UBC_NDE_FETCH_DIS|
                         XDMA_UBC_NDEN_UPDATED | usart->xfer->rx_num;
                         xdmadRxCfg.mbr_cfg = XDMAC_CC_TYPE_PER_TRAN |
                         XDMAC_CC_MBSIZE_SINGLE |
                         XDMAC_CC_DSYNC_PER2MEM |
                         XDMAC_CC_CSIZE_CHK_1 |
                         XDMAC_CC_DWIDTH_BYTE|
                         XDMAC_CC_SIF_AHB_IF1 |
                         XDMAC_CC_DIF_AHB_IF0 |
                         XDMAC_CC_SAM_FIXED_AM |
                         XDMAC_CC_DAM_INCREMENTED_AM |
                         XDMAC_CC_PERID(XDMAIF_Get_ChannelNumber(usart->id, XDMAD_TRANSFER_RX));
    xdmadRxCfg.mbr_bc = 0;
    xdmadRxCfg.mbr_sus = 0;
    xdmadRxCfg.mbr_dus = 0;
    
    // Clean cache
    SCB_CleanInvalidateDCache();
    
    if (XDMAD_ConfigureTransfer(usart->dma, *usart->dma_rx, &xdmadRxCfg, 0, 0, XDMA_INT))
      return ARM_DRIVER_ERROR;
    if (XDMAD_StartTransfer(usart->dma, *usart->dma_rx))
      return ARM_DRIVER_ERROR;
  }
  else
#endif
#if defined(USART_IRQ_RX)
  {
    // Configure and enable SPI interrupt
    NVIC_ClearPendingIRQ(usart->irq_num);
    NVIC_SetPriority(usart->irq_num,1);
    NVIC_EnableIRQ(usart->irq_num);
    usart->reg->US_IER = US_IER_RXRDY;
  }
#else
  {
    uint32_t status;

    while (num--) {
      do {
        status = usart->reg->US_CSR;
      } while ((status & US_CSR_RXRDY) == 0);
        if ((status & (US_CSR_RXBRK | US_CSR_OVRE | US_CSR_FRAME | US_CSR_PARE | US_CSR_TIMEOUT)) != 0 ) {
        usart->info->status.rx_busy = 0U;
        if (status & (US_CSR_RXBRK))
            usart->info->status.rx_break++;
        if (status & (US_CSR_OVRE))
            usart->info->status.rx_overflow++;
        if (status & (US_CSR_FRAME))
            usart->info->status.rx_framing_error++;
        if (status & (US_CSR_PARE))
            usart->info->status.rx_parity_error++;
      }

      usart->info->cb_event(ARM_USART_EVENT_RECEIVE_COMPLETE);

      usart->xfer->rx_buf[usart->xfer->rx_cnt++] = usart->reg->US_RHR;
    }

    // Set RX busy flag
    usart->info->status.rx_busy = 0U;
  }
#endif
  
  // Synchronous mode
  if (usart->info->mode == ARM_USART_MODE_SYNCHRONOUS_MASTER) {
    if (usart->xfer->sync_mode == 0U) {
      usart->xfer->sync_mode = USART_SYNC_MODE_RX;
      // Send dummy data
      stat = USARTx_Send (&usart->xfer->def_val, num, usart);
      if (stat == ARM_DRIVER_ERROR_BUSY) { return ARM_DRIVER_ERROR_BUSY; }
    }
  }

  return ARM_DRIVER_OK;
}

/**
  \fn          int32_t USARTx_Transfer (const void             *data_out,
                                             void             *data_in,
                                             uint32_t          num,
                                       const USART_RESOURCES  *usart)
  \brief       Start sending/receiving data to/from USART transmitter/receiver.
  \param[in]   data_out  Pointer to buffer with data to send to USART transmitter
  \param[out]  data_in   Pointer to buffer for data to receive from USART receiver
  \param[in]   num       Number of data items to transfer
  \param[in]   usart     Pointer to USART resources
  \return      \ref execution_status
 */
int32_t USARTx_Transfer(const void *data_out, void *data_in, uint32_t num, const USART_RESOURCES  *usart)
{
  int32_t status;

  if ((data_out == NULL) || (data_in == NULL) || (num == 0U)) {
    // Invalid parameters
    return ARM_DRIVER_ERROR_PARAMETER;
  }

  if ((usart->info->state & USART_CONFIGURED) == 0U) {
    // USART is not configured
    return ARM_DRIVER_ERROR;
  }

  if (usart->info->mode == ARM_USART_MODE_SYNCHRONOUS_MASTER) {

    // Set xfer mode
    usart->xfer->sync_mode = USART_SYNC_MODE_TX_RX;

    // Receive
    status = USARTx_Receive (data_in, num, usart);
    if (status != ARM_DRIVER_OK) { return status; }

    // Send
    status = USARTx_Send (data_out, num, usart);
    if (status != ARM_DRIVER_OK) { return status; }

  } else {
    // Only in synchronous mode
    return ARM_DRIVER_ERROR;
  }
  return ARM_DRIVER_OK;}

/**
  \fn          uint32_t USARTx_GetTxCount (const USART_RESOURCES *usart)
  \brief       Get transmitted data count.
  \param[in]   usart     Pointer to USART resources
  \return      number of data items transmitted
*/
uint32_t USARTx_GetTxCount(const USART_RESOURCES *usart)
{
  return usart->xfer->tx_cnt;
}

/**
  \fn          uint32_t USARTx_GetRxCount (const USART_RESOURCES *usart)
  \brief       Get received data count.
  \param[in]   usart     Pointer to USART resources
  \return      number of data items received
*/
uint32_t USARTx_GetRxCount(const USART_RESOURCES *usart)
{
  return usart->xfer->rx_cnt;
}

/**
  \fn          int32_t USARTx_Control (      uint32_t          control,
                                             uint32_t          arg,
                                       const USART_RESOURCES  *usart)
  \brief       Control USART Interface.
  \param[in]   control  Operation
  \param[in]   arg      Argument of operation (optional)
  \param[in]   usart    Pointer to USART resources
  \return      common \ref execution_status and driver specific \ref usart_execution_status
*/int32_t USARTx_Control(uint32_t control, uint32_t arg, const USART_RESOURCES  *usart)
{
  uint32_t mode, flow_control;
  uint32_t cr, mr;

  if ((usart->info->state & USART_POWERED) == 0U) {
    // USART not powered
    return ARM_DRIVER_ERROR;
  }

  cr  = 0U;
  mr  = 0U;
  switch (control & ARM_USART_CONTROL_Msk) {
  // Control break
  case ARM_USART_CONTROL_BREAK:
    if (arg) {
      if (usart->info->state & USART_SEND_ACTIVE) { return ARM_DRIVER_ERROR_BUSY; }

      // Set Send active and Break flag
      usart->info->state      |= USART_SEND_ACTIVE;
      usart->xfer->break_flag  = 1U;

      // Enable TX interrupt and send break
      usart->reg->US_IER |= US_IER_TXRDY;
      usart->reg->US_CR |=  US_CR_STTBRK;
    } else {
      if (usart->xfer->break_flag) {
      // Disable TX interrupt
      usart->reg->US_IER &= ~US_IER_TXRDY;

      // Clear break and Send Active flag
      usart->xfer->break_flag = 0U;
      usart->info->state     |= USART_SEND_ACTIVE;
      }
    }
    return ARM_DRIVER_OK;

  // Abort Send
  case ARM_USART_ABORT_SEND:
    // Disable TX and TC interrupt
    usart->reg->US_IDR &= ~(US_IDR_TXRDY);

    // If DMA mode - disable DMA channel
    if ((usart->dma_tx != NULL) && ((usart->info->state & USART_SEND_ACTIVE) != 0)) {
      // Abort TX DMA transfer
      XDMAD_StopTransfer(usart->dma, *usart->dma_tx);
    }

    // Clear break flag
    usart->xfer->break_flag = 0U;

    // Clear Send active flag
    usart->info->state &= ~USART_SEND_ACTIVE;
    return ARM_DRIVER_OK;

  // Abort receive
  case ARM_USART_ABORT_RECEIVE:
    // Disable RX interrupt
    usart->reg->US_IDR &= ~US_IDR_RXRDY;

    // If DMA mode - disable DMA channel
    if ((usart->dma_rx != NULL) && (usart->info->status.rx_busy != 0)) {
      // Abort TX DMA transfer
      XDMAD_StopTransfer(usart->dma, *usart->dma_rx);
    }

    // Clear RX busy status
    usart->info->status.rx_busy = 0U;
    return ARM_DRIVER_OK;

  // Abort transfer
  case ARM_USART_ABORT_TRANSFER:
    // Disable TX, TC and RX interrupt
    usart->reg->US_IDR &= ~(US_IDR_TXRDY | US_IDR_RXRDY);

    // If DMA mode - disable DMA channel
    if ((usart->dma_tx != NULL) && ((usart->info->state & USART_SEND_ACTIVE) != 0U)) {
      // Abort TX DMA transfer
      XDMAD_StopTransfer(usart->dma, *usart->dma_tx);
    }

    // If DMA mode - disable DMA channel
    if ((usart->dma_rx != NULL) && (usart->info->status.rx_busy != 0U)) {
      // Abort TX DMA transfer
      XDMAD_StopTransfer(usart->dma, *usart->dma_rx);
    }

    // Clear busy statuses
    usart->info->status.rx_busy = 0U;
    usart->info->state &= ~USART_SEND_ACTIVE;
    return ARM_DRIVER_OK;

  // Control TX
  case ARM_USART_CONTROL_TX:
    // Check if TX pin available
    if (usart->io->tx == NULL) { return ARM_DRIVER_ERROR; }
    if (arg) {
      usart->info->state |= USART_TX_ENABLED;

      // Transmitter enable
      usart->reg->US_CR |= US_CR_TXEN;
    } else {
      usart->info->state &= ~USART_TX_ENABLED;

      // Transmitter disable
      usart->reg->US_CR |= US_CR_TXDIS;
    }
    return ARM_DRIVER_OK;

  // Control RX
  case ARM_USART_CONTROL_RX:
    // Check if RX pin available
    if (usart->io->rx == NULL) { return ARM_DRIVER_ERROR; }
    if (arg) {
      usart->info->state |= USART_RX_ENABLED;

      // Transmitter enable
      usart->reg->US_CR |= US_CR_RXEN;
    } else {
      usart->info->state &= ~USART_RX_ENABLED;

      // Transmitter disable
      usart->reg->US_CR |= US_CR_RXDIS;
    }
    return ARM_DRIVER_OK;

    default: break;
  }

  // Check if busy
  if ((usart->info->status.rx_busy != 0U) || ((usart->info->state & USART_SEND_ACTIVE) != 0U)) {
    return ARM_DRIVER_ERROR_BUSY;
  }

  if (((usart->reg->US_CR & US_CR_TXEN) != 0) && ((usart->reg->US_CSR & US_CSR_TXRDY) == 0U)) {
    return ARM_DRIVER_ERROR_BUSY;
  }

  switch (control & ARM_USART_CONTROL_Msk) {
    case ARM_USART_MODE_ASYNCHRONOUS:
      break;
    case ARM_USART_MODE_SYNCHRONOUS_MASTER:
      if (usart->capabilities.synchronous_master) {
        // Enable last bit clock pulse
        mr |= US_MR_CLKO;
      } else { return ARM_USART_ERROR_MODE; }
      mode = ARM_USART_MODE_SYNCHRONOUS_MASTER;
      mr |= US_MR_SYNC;
      break;
    case ARM_USART_MODE_SYNCHRONOUS_SLAVE:
      return ARM_USART_ERROR_MODE;
    case ARM_USART_MODE_SINGLE_WIRE:
      return ARM_USART_ERROR_MODE;
    case ARM_USART_MODE_IRDA:
      return ARM_USART_ERROR_MODE;
    case ARM_USART_MODE_SMART_CARD:
      if (usart->capabilities.smart_card) {
        // Enable Smart card mode
        usart->reg->US_CR = 0x4u; // ISO7816 T=0
        //cr |= 0x5u; // ISO7816 T=1
        usart->reg->US_MR = US_MR_CLKO;
      } else { return ARM_USART_ERROR_MODE; }
      mode = ARM_USART_MODE_SMART_CARD;
      break;

    // Default TX value
    case ARM_USART_SET_DEFAULT_TX_VALUE:
      usart->xfer->def_val = (uint16_t)arg;
      return ARM_DRIVER_OK;

    // IrDA pulse
    case ARM_USART_SET_IRDA_PULSE:
      return ARM_USART_ERROR_MODE;

    // SmartCard guard time
    case ARM_USART_SET_SMART_CARD_GUARD_TIME:
      if (usart->info->mode == ARM_USART_MODE_SMART_CARD) {
        if (arg > 255U) return ARM_DRIVER_ERROR;

        usart->reg->US_TTGR = arg;
      } else { return ARM_DRIVER_ERROR; }
      return ARM_DRIVER_OK;

    // SmartCard clock
    case ARM_USART_SET_SMART_CARD_CLOCK:
      return ARM_DRIVER_ERROR_UNSUPPORTED;

    // SmartCard NACK
    case ARM_USART_CONTROL_SMART_CARD_NACK:
      /* if (usart->info->mode == ARM_USART_MODE_SMART_CARD) { */
      /*   // SmartCard NACK Enable */
      /*   if (arg != 0U) { usart->reg->US_MR |= ~US_MR_INACK; } */
      /* } else { return ARM_DRIVER_ERROR; } */
      return ARM_DRIVER_ERROR_UNSUPPORTED;

    // Unsupported command
    default:
      return ARM_DRIVER_ERROR_UNSUPPORTED;
  }

  // USART Data bits
  switch (control & ARM_USART_DATA_BITS_Msk) {
    case ARM_USART_DATA_BITS_5: mr |= US_MR_CHRL_5_BIT;    break;
    case ARM_USART_DATA_BITS_6: mr |= US_MR_CHRL_6_BIT;    break;
    case ARM_USART_DATA_BITS_7: mr |= US_MR_CHRL_7_BIT;    break;
    case ARM_USART_DATA_BITS_8: mr |= US_MR_CHRL_8_BIT;    break;
    case ARM_USART_DATA_BITS_9: mr |= US_MR_MODE9;         break;
    default: return ARM_USART_ERROR_DATA_BITS;
  }

  // USART Parity
  switch (control & ARM_USART_PARITY_Msk) {
    case ARM_USART_PARITY_NONE:   mr |= US_MR_PAR_NO;        break;
    case ARM_USART_PARITY_EVEN:   mr |= US_MR_PAR_EVEN;      break;
    case ARM_USART_PARITY_ODD:    mr |= US_MR_PAR_ODD;       break;
    default: return ARM_USART_ERROR_PARITY;
  }

  // USART Stop bits
  switch (control & ARM_USART_STOP_BITS_Msk) {
    case ARM_USART_STOP_BITS_1:   mr |= US_MR_NBSTOP_1_BIT;    break;
    case ARM_USART_STOP_BITS_2:   mr |= US_MR_NBSTOP_2_BIT;    break;
    case ARM_USART_STOP_BITS_1_5: mr |= US_MR_NBSTOP_1_5_BIT;  break;
      default: return ARM_USART_ERROR_STOP_BITS;
  }

  // USART Flow control
  switch (control & ARM_USART_FLOW_CONTROL_Msk) {
    case ARM_USART_FLOW_CONTROL_NONE:
      flow_control = ARM_USART_FLOW_CONTROL_NONE;
      break;
    case ARM_USART_FLOW_CONTROL_CTS:
      if (usart->capabilities.flow_control_rts) {
        flow_control = ARM_USART_FLOW_CONTROL_CTS;
        // CTS Enable
        //usart->reg->US_IER = US_IER_CTSIC;
      }
      else { return ARM_USART_ERROR_FLOW_CONTROL; }
      break;
    case ARM_USART_FLOW_CONTROL_RTS_CTS:
      if ((usart->capabilities.flow_control_rts != 0U) &&
          (usart->capabilities.flow_control_cts != 0U)) {
        flow_control = ARM_USART_FLOW_CONTROL_RTS_CTS;
        // RTS and CTS Enable, CTS interrupt enable
        mr |= US_MR_USART_MODE_HW_HANDSHAKING;
        // CTS Enable
        //usart->reg->US_IER = US_IER_CTSIC;
      } else { return ARM_USART_ERROR_FLOW_CONTROL; }
      break;
    default: return ARM_USART_ERROR_FLOW_CONTROL;
  }

  // USART Baudrate
  {
    unsigned int CD, FP, BaudError, ActualBaudRate, OverSamp;

    /* Reset and disable receiver & transmitter*/
    usart->reg->US_CR = US_CR_RSTRX | US_CR_RSTTX
                      | US_CR_RXDIS | US_CR_TXDIS | US_CR_RSTSTA;

    usart->reg->US_IDR = 0xFFFFFFFF;
      
    switch (usart->id) {
      case ID_USART0:
      case ID_USART1:
      case ID_USART2:
        /* Configure baudrate */
        BaudError = 10;
        OverSamp = 0;

        // Configure baud rate
        while (BaudError > 5)
        {
          CD = 0;
          FP = 0;

          CD = ((SystemCoreClock/2) / (arg * 8*(2-OverSamp)));
          FP = (((SystemCoreClock/2) / (arg * (2-OverSamp)) ) - CD * 8);
          ActualBaudRate = ((SystemCoreClock/2) /(CD*8 + FP))/(2-OverSamp);
          BaudError = (100-((arg*100/ActualBaudRate)));

          if (BaudError > 5) {
            OverSamp++;
            if(OverSamp>=2) {
              return ARM_USART_ERROR_BAUDRATE;
            }
          }
        }
        usart->reg->US_BRGR = (US_BRGR_CD(CD) | US_BRGR_FP(FP));
        mr |= OverSamp << 19;
        break;

      case ID_UART3:
      case ID_UART4:
        usart->reg->US_BRGR = (SystemCoreClock/2 / arg) / 16;
        break;
    }
  }
  // Configuration is OK - Mode is valid
  usart->info->mode = mode;

  // Save flow control mode
  usart->info->flow_control = flow_control;

  // Configure USART control registers
  usart->reg->US_CR = cr;
  usart->reg->US_MR = mr | US_MR_CHMODE_NORMAL | US_MR_USCLKS_MCK;

  // Set configured flag
  usart->info->state |= USART_CONFIGURED;

  return ARM_DRIVER_OK;
}

/**
  \fn          ARM_USART_STATUS USARTx_GetStatus (const USART_RESOURCES *usart)
  \brief       Get USART status.
  \param[in]   usart     Pointer to USART resources
  \return      USART status \ref ARM_USART_STATUS
*/
ARM_USART_STATUS USARTx_GetStatus(const USART_RESOURCES *usart)
{
  usart->info->status.tx_busy = ((usart->reg->US_CSR & US_CSR_TXRDY) ? (0U) : (1U));
  usart->info->status.rx_busy = ((usart->reg->US_CSR & US_CSR_RXRDY) ? (0U) : (1U));

  return usart->info->status;
}

/**
  \fn          int32_t USARTx_SetModemControl (      ARM_USART_MODEM_CONTROL  control,
                                               const USART_RESOURCES         *usart)
  \brief       Set USART Modem Control line state.
  \param[in]   control   \ref ARM_USART_MODEM_CONTROL
  \param[in]   usart     Pointer to USART resources
  \return      \ref execution_status
*/
int32_t USARTx_SetModemControl(ARM_USART_MODEM_CONTROL control, const USART_RESOURCES *usart)
{
  switch (control) {
    case ARM_USART_RTS_CLEAR:
      if ((usart->info->flow_control == ARM_USART_FLOW_CONTROL_NONE) ||
          (usart->info->flow_control == ARM_USART_FLOW_CONTROL_CTS)) {
        PIO_Set(usart->io->rts);
      } else {
        // Hardware RTS
        return ARM_DRIVER_ERROR;
      }
      break;
    case ARM_USART_RTS_SET:
      if ((usart->info->flow_control == ARM_USART_FLOW_CONTROL_NONE) ||
          (usart->info->flow_control == ARM_USART_FLOW_CONTROL_CTS)) {
        PIO_Clear(usart->io->rts);
      } else {
        // Hardware RTS
        return ARM_DRIVER_ERROR;
      }
      break;
    case ARM_USART_DTR_CLEAR:
    case ARM_USART_DTR_SET:
    default: return ARM_DRIVER_ERROR;
  }

  return ARM_DRIVER_OK;
}

/**
  \fn          ARM_USART_MODEM_STATUS USARTx_GetModemStatus (const USART_RESOURCES *usart)
  \brief       Get USART Modem Status lines state.
  \param[in]   usart     Pointer to USART resources
  \return      modem status \ref ARM_USART_MODEM_STATUS
*/
ARM_USART_MODEM_STATUS USARTx_GetModemStatus(const USART_RESOURCES *usart)
{
  ARM_USART_MODEM_STATUS modem_status;

  if ((usart->info->flow_control == ARM_USART_FLOW_CONTROL_NONE) ||
      (usart->info->flow_control == ARM_USART_FLOW_CONTROL_RTS)) {
    if (PIO_Get(usart->io->cts) == 0) {
      modem_status.cts = 1U;
    } else {
      modem_status.cts = 0U;
    }
  } else {
    // Hardware CTS
    modem_status.cts = 0U;
  }
  modem_status.dsr = 0U;
  modem_status.ri  = 0U;
  modem_status.dcd = 0U;

  return modem_status;
}

void ARM_USART_SignalEvent(uint32_t event)
{
    // function body
}

// End USART Interface

#if (RTE_USART0 == 1)
// USART0 Driver Wrapper functions
static ARM_USART_CAPABILITIES  USART0_GetCapabilities (void)                                                { return USARTx_GetCapabilities (&USART0_Resources); }
static int32_t                 USART0_Initialize      (ARM_USART_SignalEvent_t cb_event)                    { return USARTx_Initialize (cb_event, &USART0_Resources); }
static int32_t                 USART0_Uninitialize    (void)                                                { return USARTx_Uninitialize (&USART0_Resources); }
static int32_t                 USART0_PowerControl    (ARM_POWER_STATE state)                               { return USARTx_PowerControl (state, &USART0_Resources); }
static int32_t                 USART0_Send            (const void *data, uint32_t num)                      { return USARTx_Send (data, num, &USART0_Resources); }
static int32_t                 USART0_Receive         (void *data, uint32_t num)                            { return USARTx_Receive (data, num, &USART0_Resources); }
static int32_t                 USART0_Transfer        (const void *data_out, void *data_in, uint32_t num)   { return USARTx_Transfer (data_out, data_in, num, &USART0_Resources); }
static uint32_t                USART0_GetTxCount      (void)                                                { return USARTx_GetTxCount (&USART0_Resources); }
static uint32_t                USART0_GetRxCount      (void)                                                { return USARTx_GetRxCount (&USART0_Resources); }
static int32_t                 USART0_Control         (uint32_t control, uint32_t arg)                      { return USARTx_Control (control, arg, &USART0_Resources); }
static ARM_USART_STATUS        USART0_GetStatus       (void)                                                { return USARTx_GetStatus (&USART0_Resources); }
static int32_t                 USART0_SetModemControl (ARM_USART_MODEM_CONTROL control)                     { return USARTx_SetModemControl (control, &USART0_Resources); }
static ARM_USART_MODEM_STATUS  USART0_GetModemStatus  (void)                                                { return USARTx_GetModemStatus (&USART0_Resources); }
       void                    USART0_Handler      (void)                                                {        USARTx_IRQHandler (&USART0_Resources); }


// USART0 Driver Control Block
ARM_DRIVER_USART Driver_USART0 = {
  USARTx_GetVersion,
  USART0_GetCapabilities,
  USART0_Initialize,
  USART0_Uninitialize,
  USART0_PowerControl,
  USART0_Send,
  USART0_Receive,
  USART0_Transfer,
  USART0_GetTxCount,
  USART0_GetRxCount,
  USART0_Control,
  USART0_GetStatus,
  USART0_SetModemControl,
  USART0_GetModemStatus
};
#endif /* #if RTE_USART0 */

#if (RTE_USART1 == 1)
// USART1 Driver Wrapper functions
static ARM_USART_CAPABILITIES  USART1_GetCapabilities (void)                                                { return USARTx_GetCapabilities (&USART1_Resources); }
static int32_t                 USART1_Initialize      (ARM_USART_SignalEvent_t cb_event)                    { return USARTx_Initialize (cb_event, &USART1_Resources); }
static int32_t                 USART1_Uninitialize    (void)                                                { return USARTx_Uninitialize (&USART1_Resources); }
static int32_t                 USART1_PowerControl    (ARM_POWER_STATE state)                               { return USARTx_PowerControl (state, &USART1_Resources); }
static int32_t                 USART1_Send            (const void *data, uint32_t num)                      { return USARTx_Send (data, num, &USART1_Resources); }
static int32_t                 USART1_Receive         (void *data, uint32_t num)                            { return USARTx_Receive (data, num, &USART1_Resources); }
static int32_t                 USART1_Transfer        (const void *data_out, void *data_in, uint32_t num)   { return USARTx_Transfer (data_out, data_in, num, &USART1_Resources); }
static uint32_t                USART1_GetTxCount      (void)                                                { return USARTx_GetTxCount (&USART1_Resources); }
static uint32_t                USART1_GetRxCount      (void)                                                { return USARTx_GetRxCount (&USART1_Resources); }
static int32_t                 USART1_Control         (uint32_t control, uint32_t arg)                      { return USARTx_Control (control, arg, &USART1_Resources); }
static ARM_USART_STATUS        USART1_GetStatus       (void)                                                { return USARTx_GetStatus (&USART1_Resources); }
static int32_t                 USART1_SetModemControl (ARM_USART_MODEM_CONTROL control)                     { return USARTx_SetModemControl (control, &USART1_Resources); }
static ARM_USART_MODEM_STATUS  USART1_GetModemStatus  (void)                                                { return USARTx_GetModemStatus (&USART1_Resources); }
       void                    USART1_Handler      (void)                                                {        USARTx_IRQHandler (&USART1_Resources); }


// USART1 Driver Control Block
ARM_DRIVER_USART Driver_USART1 = {
  USARTx_GetVersion,
  USART1_GetCapabilities,
  USART1_Initialize,
  USART1_Uninitialize,
  USART1_PowerControl,
  USART1_Send,
  USART1_Receive,
  USART1_Transfer,
  USART1_GetTxCount,
  USART1_GetRxCount,
  USART1_Control,
  USART1_GetStatus,
  USART1_SetModemControl,
  USART1_GetModemStatus
};
#endif /* #if RTE_USART1 */

#if (RTE_USART2 == 1)
// USART1 Driver Wrapper functions
static ARM_USART_CAPABILITIES  USART2_GetCapabilities (void)                                                { return USARTx_GetCapabilities (&USART2_Resources); }
static int32_t                 USART2_Initialize      (ARM_USART_SignalEvent_t cb_event)                    { return USARTx_Initialize (cb_event, &USART2_Resources); }
static int32_t                 USART2_Uninitialize    (void)                                                { return USARTx_Uninitialize (&USART2_Resources); }
static int32_t                 USART2_PowerControl    (ARM_POWER_STATE state)                               { return USARTx_PowerControl (state, &USART2_Resources); }
static int32_t                 USART2_Send            (const void *data, uint32_t num)                      { return USARTx_Send (data, num, &USART2_Resources); }
static int32_t                 USART2_Receive         (void *data, uint32_t num)                            { return USARTx_Receive (data, num, &USART2_Resources); }
static int32_t                 USART2_Transfer        (const void *data_out, void *data_in, uint32_t num)   { return USARTx_Transfer (data_out, data_in, num, &USART2_Resources); }
static uint32_t                USART2_GetTxCount      (void)                                                { return USARTx_GetTxCount (&USART2_Resources); }
static uint32_t                USART2_GetRxCount      (void)                                                { return USARTx_GetRxCount (&USART2_Resources); }
static int32_t                 USART2_Control         (uint32_t control, uint32_t arg)                      { return USARTx_Control (control, arg, &USART2_Resources); }
static ARM_USART_STATUS        USART2_GetStatus       (void)                                                { return USARTx_GetStatus (&USART2_Resources); }
static int32_t                 USART2_SetModemControl (ARM_USART_MODEM_CONTROL control)                     { return USARTx_SetModemControl (control, &USART2_Resources); }
static ARM_USART_MODEM_STATUS  USART2_GetModemStatus  (void)                                                { return USARTx_GetModemStatus (&USART2_Resources); }
       void                    USART2_Handler         (void)                                                {        USARTx_IRQHandler (&USART2_Resources); }


// USART2 Driver Control Block
ARM_DRIVER_USART Driver_USART2 = {
  USARTx_GetVersion,
  USART2_GetCapabilities,
  USART2_Initialize,
  USART2_Uninitialize,
  USART2_PowerControl,
  USART2_Send,
  USART2_Receive,
  USART2_Transfer,
  USART2_GetTxCount,
  USART2_GetRxCount,
  USART2_Control,
  USART2_GetStatus,
  USART2_SetModemControl,
  USART2_GetModemStatus
};
#endif /* #if RTE_USART2 */
