/* -----------------------------------------------------------------------------
 * Copyright (c) 2015 ARM Ltd.
 *  
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *  
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 * 
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * $Date:        30. April 2015
 * $Revision:    V1.02
 *
 * Project:      CMSIS USART Driver for Atmel SAM-ESV7
 * -------------------------------------------------------------------------- */
 
#ifndef __USART_SAM_ESV7_H
#define __USART_SAM_ESV7_H

#include "RTE_Device.h"
#include "RTE_Components.h"
#include "Driver_USART.h"
#include "chip.h"
#include <string.h>

#if (defined(RTE_Drivers_USART) && \
     (RTE_USART0 == 0) && \
     (RTE_USART1 == 0) && \
     (RTE_USART2 == 0))
#error "USART not configured in RTE_Device.h!"
#endif

// USART Input/Output Configuration
typedef const struct _USART_IO {
    Pin                  *tx;             // TX   Pin identifier
    Pin                  *rx;             // RX   Pin identifier
    Pin                  *sck;            // CLK  Pin identifier
    Pin                  *rts;            // RTS  Pin identifier
    Pin                  *cts;            // CTS  Pin identifier
    Pin                  *dcd;            // DCD  Pin identifier
    Pin                  *dsr;            // DSR  Pin identifier
    Pin                  *dtr;            // DTR  Pin identifier
    Pin                  *ri;             // RI   Pin identifier
    Pin                  *lcol;           // LCOL Pin identifier
    
} USART_IO;

// USART Transfer Information (Run-Time)
typedef struct _USART_TRANSFER_INFO {
  uint32_t              rx_num;         // Total number of receive data
  uint32_t              tx_num;         // Total number of transmit data
  uint8_t              *rx_buf;         // Pointer to in data buffer
  uint8_t              *tx_buf;         // Pointer to out data buffer
  uint32_t              rx_cnt;         // Number of data received
  uint32_t              tx_cnt;         // Number of data sent
  uint16_t              dump_val;       // Variable for dumping DMA data
  uint16_t              def_val;        // Default transfer value
  uint32_t              sync_mode;      // Synchronous mode flag
  uint8_t               break_flag;     // Transmit break flag
} USART_TRANSFER_INFO;

// USART Information (Run-time)
typedef struct _USART_INFO {
  ARM_USART_SignalEvent_t cb_event;            // Event Callback
  ARM_USART_STATUS        status;              // Status flags
  uint8_t                 state;               // Current USART state
  uint32_t                mode;                // Current USART mode
  uint32_t                flow_control;        // Flow control
} USART_INFO;

// USART Resources definition
typedef struct {
  ARM_USART_CAPABILITIES  capabilities;        // Capabilities

  Usart                  *reg;                 // USART peripheral pointer
  uint32_t                id;                  // Peripheral id
  IRQn_Type               irq_num;             // SPI IRQ Number
  USART_IO               *io;                  // USART Input/Output pins
  USART_INFO             *info;                // Run-Time Information
  USART_TRANSFER_INFO    *xfer;                // USART transfer information
  sXdmad     *dma;           // DMA driver instance 
  uint32_t     *dma_tx;        // DMA TX channel  
  uint32_t     *dma_rx;        // DMA RX channel
} USART_RESOURCES;

// Current driver status flag definition
#define USART_INITIALIZED      ((uint8_t)(1U))
#define USART_POWERED          ((uint8_t)(1U << 1))
#define USART_CONFIGURED       ((uint8_t)(1U << 2))
#define USART_TX_ENABLED       ((uint8_t)(1U << 3))
#define USART_RX_ENABLED       ((uint8_t)(1U << 4))
#define USART_SEND_ACTIVE      ((uint8_t)(1U << 5))

// USART synchronous xfer modes
#define USART_SYNC_MODE_TX           ( 1UL )
#define USART_SYNC_MODE_RX           ( 2UL )
#define USART_SYNC_MODE_TX_RX        (USART_SYNC_MODE_TX | \
                                      USART_SYNC_MODE_RX)

#endif /* __USART_SAM_ESV7_H */
